@@ -1,5 +1,5 @@
 /*
- * $Id: addressbook_popup.c 6384 2008-06-16 03:22:25Z ajc $
+ * $Id: addressbook_popup.c 7087 2009-02-10 18:50:50Z dothebart $
  *
  * AJAX-powered auto-completion
  */
@@ -28,12 +28,16 @@ void display_address_book_middle_div(void) {
 	char buf[256];
 	long len;
 	char *Name;
+	const char *VCName;
 	void *Namee;
+	StrBuf *DefAddrBook;
 	HashList *List;
 	HashPos  *it;
 
 	begin_ajax_response();
 
+	DefAddrBook = get_room_pref("defaddrbook");
+
 	wprintf("<table border=0 width=100%%><tr valign=middle>");
 	wprintf("<td align=left><img src=\"static/viewcontacts_32x.gif\"></td>");
 	wprintf("<td align=center>");
@@ -44,8 +48,10 @@ void display_address_book_middle_div(void) {
 		bstr("target_input")
 	);
 
-	wprintf("<option value=\"__LOCAL_USERS__\">");
-	escputs(serv_info.serv_humannode);
+	wprintf("<option value=\"__LOCAL_USERS__\" %s>", 
+		(strcmp(ChrPtr(DefAddrBook), "__LOCAL_USERS__") == 0)?
+		"selected=\"selected\" ":"");
+	escputs(ChrPtr(WC->serv_info->serv_humannode));
 	wprintf("</option>\n");
 
 	
@@ -61,11 +67,14 @@ void display_address_book_middle_div(void) {
 	}
 
 	SortByHashKey(List, 1);
-	it = GetNewHashPos();
-	while (GetNextHashPos(List, it, &len, &Name, &Namee)) {
+	it = GetNewHashPos(List, 0);
+	while (GetNextHashPos(List, it, &len, &VCName, &Namee)) {
 		wprintf("<option value=\"");
 		urlescputs((char*)Namee);
-		wprintf("\">");
+		if (strcmp(ChrPtr(DefAddrBook), Namee) == 0)
+			wprintf("\" selected=\"selected\" >");
+		else
+			wprintf("\">");
 		escputs((char*)Namee);
 		wprintf("</option>\n");
 	}
@@ -79,9 +88,7 @@ void display_address_book_middle_div(void) {
 		"><img src=\"static/closewindow.gif\">");
 	wprintf("</td></tr></table>");
 
-	wprintf("<script type=\"text/javascript\">"
-		"PopulateAddressBookInnerDiv($('which_addr_book').value,'%s');"
-		"</script>\n",
+	wprintf("<script type=\"text/javascript\">PopulateAddressBookInnerDiv($('which_addr_book').value,'%s');</script>",
 		bstr("target_input")
 	);
 
@@ -100,11 +107,12 @@ void display_address_book_inner_div() {
 	char target_label[64];
 	long len;
 	char *Name;
+	const char *VCName;
 	void *Namee;
 	HashList *List;
 	HashPos  *it;
 	int i;
-	char saved_roomname[128];
+	StrBuf *saved_roomname;
 
 	begin_ajax_response();
 
@@ -118,12 +126,16 @@ void display_address_book_inner_div() {
 		if (buf[0] == '1') while(len = serv_getln(buf, sizeof buf), strcmp(buf, "000")) {
 			Name = (char*) malloc(len + 1);
 			len = extract_token(Name, buf, 0, '|', len + 1);
+			if((len > 5) && (strncmp(Name, "SYS_", 4) == 0)) {
+				free(Name);
+				continue;
+			}
 			Put(List, Name, len, Name, NULL);
 
 		}
 		SortByHashKey(List, 1);
-		it = GetNewHashPos();
-		while (GetNextHashPos(List, it, &len, &Name, &Namee)) {
+		it = GetNewHashPos(List, 0);
+		while (GetNextHashPos(List, it, &len, &VCName, &Namee)) {
 			wprintf("<option value=\"");
 			escputs((char*)Namee);
 			wprintf("\">");
@@ -135,8 +147,9 @@ void display_address_book_inner_div() {
 	}
 
 	else {
-		safestrncpy(saved_roomname, WC->wc_roomname, sizeof saved_roomname);
-		gotoroom(bstr("which_addr_book"));
+		set_room_pref("defaddrbook",NewStrBufDup(sbstr("which_addr_book")), 0);
+		saved_roomname = NewStrBufDup(WC->wc_roomname);
+		gotoroom(sbstr("which_addr_book"));
 		serv_puts("DVCA");
 		serv_getln(buf, sizeof buf);
 		if (buf[0] == '1') while(len = serv_getln(buf, sizeof buf), strcmp(buf, "000")) {
@@ -146,8 +159,8 @@ void display_address_book_inner_div() {
 
 		}
 		SortByHashKey(List, 1);
-		it = GetNewHashPos();
-		while (GetNextHashPos(List, it, &len, &Name, (void**)&Namee)) {
+		it = GetNewHashPos(List, 0);
+		while (GetNextHashPos(List, it, &len, &VCName, (void**)&Namee)) {
 			wprintf("<option value=\"");
 			escputs((char*)Namee);
 			wprintf("\">");
@@ -157,6 +170,7 @@ void display_address_book_inner_div() {
 		DeleteHashPos(&it);
 		DeleteHash(&List);
 		gotoroom(saved_roomname);
+		FreeStrBuf(&saved_roomname);
 	}
 
 	wprintf("</select>\n");
@@ -182,3 +196,14 @@ void display_address_book_inner_div() {
 
 	end_ajax_response();
 }
+
+
+
+
+void 
+InitModule_ADDRBOOK_POPUP
+(void)
+{
+	WebcitAddUrlHandler(HKEY("display_address_book_middle_div"), display_address_book_middle_div, 0);
+	WebcitAddUrlHandler(HKEY("display_address_book_inner_div"), display_address_book_inner_div, 0);
+}
