@@ -1,5 +1,5 @@
 /*
- * $Id: auth.c 6374 2008-06-07 04:44:11Z ajc $
+ * $Id: auth.c 7143 2009-02-21 14:12:43Z dothebart $
  *
  * WebcitAuth; Handles authentication of users to a Citadel server.
  */
@@ -8,6 +8,9 @@
 #include "webserver.h"
 #include <ctype.h>
 
+
+void display_reg(int during_login);
+
 /*
  * Access level definitions.  This is initialized from a function rather than a
  * static array so that the strings may be localized.
@@ -31,72 +34,12 @@ void initialize_axdefs(void) {
  * Display the login screen
  * mesg = the error message if last attempt failed.
  */
-void display_login(char *mesg)
+void display_login(void)
 {
-	char buf[SIZ];
-
-	output_headers(1, 1, 2, 0, 0, 0);
-	wprintf("<div id=\"login_screen\">\n");
-
-	if (mesg != NULL) if (!IsEmptyStr(mesg)) {
-			stresc(buf, SIZ,  mesg, 0, 0);
-			svprintf(HKEY("MESG"), WCS_STRING, "%s", buf);
-	}
-
-	svprintf(HKEY("LOGIN_INSTRUCTIONS"), WCS_STRING,
-		_("<ul>"
-		"<li><b>If you already have an account on %s</b>, "
-		"enter your user name and password and click &quot;Login.&quot; "
-		"<li><b>If you are a new user</b>, enter the name and password "
-		"you wish to use, "
-		"and click &quot;New User.&quot; "
-		"<li>Please log off properly when finished. "
-		"<li>You must use a browser that supports <i>frames</i> and "
-		"<i>cookies</i>. "
-		"<li>Also keep in mind that if your browser is "
-		"configured to block pop-up windows, you will not be able "
-		"to receive any instant messages.<br />"
-		"</ul>"),
-		serv_info.serv_humannode
-	);
-
-	svput("USERNAME_BOX", WCS_STRING, _("User name:"));
-	svput("PASSWORD_BOX", WCS_STRING, _("Password:"));
-	svput("LANGUAGE_BOX", WCS_STRING, _("Language:"));
-	svput("LOGIN_BUTTON", WCS_STRING, _("Login"));
-	svput("NEWUSER_BUTTON", WCS_STRING, _("New User"));
-	svput("EXIT_BUTTON", WCS_STRING, _("Exit"));
-	svput("HELLO", WCS_SERVCMD, "MESG hello");
-	svprintf(HKEY("BOXTITLE"), WCS_STRING, _("%s - powered by <a href=\"http://www.citadel.org\">Citadel</a>"),
-		serv_info.serv_humannode);
-	svcallback("DO_LANGUAGE_BOX", offer_languages);
-	if (serv_info.serv_newuser_disabled) {
-		svput("NEWUSER_BUTTON_PRE", WCS_STRING, "<div style=\"display:none;\">");
-		svput("NEWUSER_BUTTON_POST", WCS_STRING, "</div>");
-	}
-	else {
-		svput("NEWUSER_BUTTON_PRE", WCS_STRING, "");
-		svput("NEWUSER_BUTTON_POST", WCS_STRING, "");
-	}
-
-	if (serv_info.serv_supports_openid) {
-		svprintf(HKEY("OFFER_OPENID_LOGIN"), WCS_STRING,
-			"<div align=center>"
-			"<a href=\"display_openid_login\">"
-			"<img src=\"static/openid-small.gif\" border=0 valign=middle>"
-			"%s</a>"
-			"</div>"
-			,
-			"Log in using OpenID"
-		);
-	}
-	else {
-		svput("OFFER_OPENID_LOGIN", WCS_STRING, "");
-	}
-
-	do_template("login");
-
-	wDumpContent(2);
+	begin_burst();
+	output_headers(1, 0, 0, 0, 1, 0);
+	do_template("login", NULL);
+	end_burst();
 }
 
 
@@ -108,69 +51,34 @@ void display_login(char *mesg)
  */
 void display_openid_login(char *mesg)
 {
-	char buf[SIZ];
-
-	output_headers(1, 1, 2, 0, 0, 0);
-	wprintf("<div id=\"login_screen\">\n");
-
-	if (mesg != NULL) if (!IsEmptyStr(mesg)) {
-			stresc(buf, SIZ,  mesg, 0, 0);
-			svprintf(HKEY("MESG"), WCS_STRING, "%s", buf);
-	}
-
-	svprintf(HKEY("LOGIN_INSTRUCTIONS"), WCS_STRING,
-		_("<ul>"
-		"<li>Enter your OpenID URL and click &quot;Login&quot;."
-		"<li><a href=\"http://www.citadel.org/doku.php/documentation:openid\">"
-		"Click here to learn what OpenID is and how Citadel is using it.</a>"
-		"<li>Please log off properly when finished. "
-		"<li>You must use a browser that supports <i>frames</i> and "
-		"<i>cookies</i>. "
-		"<li>Also keep in mind that if your browser is "
-		"configured to block pop-up windows, you will not be able "
-		"to receive any instant messages.<br />"
-		"</ul>")
-	);
-
-	svput("OPENID_BOX", WCS_STRING, _("OpenID URL:"));
-	svput("LANGUAGE_BOX", WCS_STRING, _("Language:"));
-	svput("LOGIN_BUTTON", WCS_STRING, _("Login"));
-	svput("EXIT_BUTTON", WCS_STRING, _("Exit"));
-	svput("HELLO", WCS_SERVCMD, "MESG hello");
-	svprintf(HKEY("BOXTITLE"), WCS_STRING, _("%s - powered by <a href=\"http://www.citadel.org\">Citadel</a>"),
-		serv_info.serv_humannode);
-	svcallback("DO_LANGUAGE_BOX", offer_languages);
-
-	svprintf(HKEY("OFFER_CONVENTIONAL_LOGIN"), WCS_STRING,
-		"<div align=center>"
-		"<a href=\"display_login\">"
-		"%s</a>"
-		"</div>"
-		,
-		"Log in using a user name and password"
-	);
-
-	do_template("openid_login");
-	wDumpContent(2);
+  begin_burst();
+  output_headers(1, 0, 0, 0, 1, 0);
+  do_template("openid_login", NULL);
+  end_burst();
 }
 
 
-void display_openid_name_request(char *claimed_id, char *username) {
-	char buf[SIZ];
+void display_openid_name_request(const StrBuf *claimed_id, const StrBuf *username) 
+{
+	StrBuf *Buf = NULL;
 
 	output_headers(1, 1, 2, 0, 0, 0);
 	wprintf("<div id=\"login_screen\">\n");
 
-	stresc(buf, sizeof buf, claimed_id, 0, 0);
+	Buf = NewStrBufPlain(NULL, StrLength(claimed_id));
+	StrEscAppend(Buf, claimed_id, NULL, 0, 0);
 	svprintf(HKEY("VERIFIED"), WCS_STRING, _("Your OpenID <tt>%s</tt> was successfully verified."),
-		claimed_id);
-	svput("CLAIMED_ID", WCS_STRING, claimed_id);
+		 ChrPtr(Buf));
+	SVPutBuf("CLAIMED_ID", Buf, 0);
 
 
-	if (!IsEmptyStr(username)) {
-		stresc(buf, sizeof buf, username, 0, 0);
-		svprintf(HKEY("REASON"), WCS_STRING,
-			_("However, the user name '%s' conflicts with an existing user."), username);
+	if (StrLength(username) > 0) {
+			Buf = NewStrBufPlain(NULL, StrLength(username));
+			StrEscAppend(Buf, claimed_id, NULL, 0, 0);
+			svprintf(HKEY("REASON"), WCS_STRING,
+				 _("However, the user name '%s' conflicts with an existing user."), 
+				 ChrPtr(Buf));
+			FreeStrBuf(&Buf);
 	}
 	else {
 		svput("REASON", WCS_STRING, "");
@@ -183,9 +91,9 @@ void display_openid_name_request(char *claimed_id, char *username) {
 	svput("EXIT_BUTTON", WCS_STRING, _("Exit"));
 
 	svprintf(HKEY("BOXTITLE"), WCS_STRING, _("%s - powered by <a href=\"http://www.citadel.org\">Citadel</a>"),
-		serv_info.serv_humannode);
+		 ChrPtr(WC->serv_info->serv_humannode));
 
-	do_template("openid_manual_create");
+	do_template("openid_manual_create", NULL);
 	wDumpContent(2);
 }
 
@@ -203,17 +111,37 @@ void display_openid_name_request(char *claimed_id, char *username) {
  * pass			his password
  * serv_response	The parameters returned from a Citadel USER or NEWU command
  */
-void become_logged_in(char *user, char *pass, char *serv_response)
+void become_logged_in(const StrBuf *user, const StrBuf *pass, StrBuf *serv_response)
 {
+	wcsession *WCC = WC;
 	char buf[SIZ];
+	StrBuf *FloorDiv;
+	StrBuf *Language = NULL;
 
 	WC->logged_in = 1;
-	extract_token(WC->wc_fullname, &serv_response[4], 0, '|', sizeof WC->wc_fullname);
-	safestrncpy(WC->wc_username, user, sizeof WC->wc_username);
-	safestrncpy(WC->wc_password, pass, sizeof WC->wc_password);
-	WC->axlevel = extract_int(&serv_response[4], 1);
-	if (WC->axlevel >= 6) {
-		WC->is_aide = 1;
+
+	if (WCC->wc_fullname == NULL)
+		WCC->wc_fullname = NewStrBufPlain(NULL, StrLength(serv_response));
+	StrBufExtract_token(WCC->wc_fullname, serv_response, 0, '|');
+	StrBufCutLeft(WCC->wc_fullname, 4 );
+	
+	if (WCC->wc_username == NULL)
+		WCC->wc_username = NewStrBufDup(user);
+	else {
+		FlushStrBuf(WCC->wc_username);
+		StrBufAppendBuf(WCC->wc_username, user, 0);
+	}
+
+	if (WCC->wc_password == NULL)
+		WCC->wc_password = NewStrBufDup(pass);
+	else {
+		FlushStrBuf(WCC->wc_password);
+		StrBufAppendBuf(WCC->wc_password, pass, 0);
+	}
+
+	WCC->axlevel = StrBufExtract_int(serv_response, 1, '|');
+	if (WCC->axlevel >= 6) { /* TODO: make this a define, else it might trick us later */
+		WCC->is_aide = 1;
 	}
 
 	load_preferences();
@@ -226,11 +154,17 @@ void become_logged_in(char *user, char *pass, char *serv_response)
 		WC->need_vali = extract_int(&buf[4], 2);
 		extract_token(WC->cs_inet_email, &buf[4], 3, '|', sizeof WC->cs_inet_email);
 	}
-
-	get_preference("current_iconbar", buf, sizeof buf);
-	WC->current_iconbar = atoi(buf);
-
-	get_preference("floordiv_expanded", WC->floordiv_expanded, sizeof WC->floordiv_expanded);
+	if (havebstr("language"))
+		set_preference("language", NewStrBufDup(SBSTR("language")), 1);
+	else {
+		get_preference("language", &Language);
+		if (Language != NULL) {
+			set_selected_language(ChrPtr(Language));
+			go_selected_language();		/* set locale */
+		}
+	}
+	get_preference("floordiv_expanded", &FloorDiv);
+	WC->floordiv_expanded = FloorDiv;
 }
 
 
@@ -239,7 +173,8 @@ void become_logged_in(char *user, char *pass, char *serv_response)
  */
 void do_login(void)
 {
-	char buf[SIZ];
+	wcsession *WCC = WC;
+	StrBuf *Buf;
 
 	if (havebstr("language")) {
 		set_selected_language(bstr("language"));
@@ -250,51 +185,76 @@ void do_login(void)
 		do_logout();
 		return;
 	}
+	Buf = NewStrBuf();
 	if (havebstr("login_action")) {
 		serv_printf("USER %s", bstr("name"));
-		serv_getln(buf, sizeof buf);
-		if (buf[0] == '3') {
+		StrBuf_ServGetln(Buf);
+		if (GetServerStatus(Buf, NULL) == 3) {
 			serv_printf("PASS %s", bstr("pass"));
-			serv_getln(buf, sizeof buf);
-			if (buf[0] == '2') {
-				become_logged_in(bstr("name"), bstr("pass"), buf);
+			StrBuf_ServGetln(Buf);
+			if (GetServerStatus(Buf, NULL) == 2) {
+				become_logged_in(sbstr("name"), sbstr("pass"), Buf);
 			} else {
-				display_login(&buf[4]);
+				snprintf(WCC->ImportantMessage, 
+					 sizeof (WCC->ImportantMessage), 
+					 "%s", 
+					 &(ChrPtr(Buf))[4]);
+				display_login();
+				FreeStrBuf(&Buf);
 				return;
 			}
 		} else {
-			display_login(&buf[4]);
+			snprintf(WCC->ImportantMessage, 
+				 sizeof (WCC->ImportantMessage), 
+				 "%s", 
+				 &(ChrPtr(Buf))[4]);
+			display_login();
+			FreeStrBuf(&Buf);
 			return;
 		}
 	}
 	if (havebstr("newuser_action")) {
 		if (!havebstr("pass")) {
-			display_login(_("Blank passwords are not allowed."));
+			snprintf(WCC->ImportantMessage, 
+				 sizeof (WCC->ImportantMessage), 
+				 "%s", 
+				 _("Blank passwords are not allowed."));
+			display_login();
+			FreeStrBuf(&Buf);
 			return;
 		}
 		serv_printf("NEWU %s", bstr("name"));
-		serv_getln(buf, sizeof buf);
-		if (buf[0] == '2') {
-			become_logged_in(bstr("name"), bstr("pass"), buf);
+		StrBuf_ServGetln(Buf);
+		if (GetServerStatus(Buf, NULL) == 2) {
+			become_logged_in(sbstr("name"), sbstr("pass"), Buf);
 			serv_printf("SETP %s", bstr("pass"));
-			serv_getln(buf, sizeof buf);
+			StrBuf_ServGetln(Buf); /* Don't care? */
 		} else {
-			display_login(&buf[4]);
+			snprintf(WCC->ImportantMessage, 
+				 sizeof (WCC->ImportantMessage), 
+				 "%s", 
+				 &(ChrPtr(Buf))[4]);
+			display_login();
+			FreeStrBuf(&Buf);
 			return;
 		}
 	}
-	if (WC->logged_in) {
-		if (WC->need_regi) {
+	if (WCC->logged_in) {
+		if (WCC->need_regi) {
 			display_reg(1);
-		} else if (WC->need_vali) {
+		} else if (WCC->need_vali) {
 			validate();
 		} else {
 			do_welcome();
 		}
 	} else {
-		display_login(_("Your password was not accepted."));
+		snprintf(WCC->ImportantMessage, 
+			 sizeof (WCC->ImportantMessage), 
+			 "%s", 
+			 _("Your password was not accepted."));
+		display_login();
 	}
-
+	FreeStrBuf(&Buf);
 }
 
 /* 
@@ -302,21 +262,28 @@ void do_login(void)
  */
 void openid_manual_create(void)
 {
+	StrBuf *Buf;
+
 	if (havebstr("exit_action")) {
 		do_logout();
 		return;
 	}
 
-	char buf[1024];
 	if (havebstr("newuser_action")) {
+		Buf = NewStrBuf();
 		serv_printf("OIDC %s", bstr("name"));
-		serv_getln(buf, sizeof buf);
-		if (buf[0] == '2') {
-			char gpass[1024] = "";
+		StrBuf_ServGetln(Buf);
+		if (GetServerStatus(Buf, NULL) == 2) {
+			StrBuf *gpass;
+
+			gpass = NewStrBuf();
 			serv_puts("SETP GENERATE_RANDOM_PASSWORD");
-			serv_getln(gpass, sizeof gpass);
-			become_logged_in(bstr("name"), &gpass[4], buf);
+			StrBuf_ServGetln(gpass);
+			StrBufCutLeft(gpass, 4);
+			become_logged_in(sbstr("name"), gpass, Buf);
+			FreeStrBuf(&gpass);
 		}
+		FreeStrBuf(&Buf);
 	}
 
 	if (WC->logged_in) {
@@ -328,7 +295,7 @@ void openid_manual_create(void)
 			do_welcome();
 		}
 	} else {
-		display_openid_name_request(bstr("openid_url"), bstr("name"));
+		display_openid_name_request(sbstr("openid_url"), sbstr("name"));
 	}
 
 }
@@ -340,6 +307,7 @@ void openid_manual_create(void)
  */
 void do_openid_login(void)
 {
+	wcsession *WCC = WC;
 	char buf[4096];
 
 	if (havebstr("language")) {
@@ -355,8 +323,8 @@ void do_openid_login(void)
 		snprintf(buf, sizeof buf,
 			"OIDS %s|%s://%s/finalize_openid_login|%s://%s",
 			bstr("openid_url"),
-			(is_https ? "https" : "http"), WC->http_host,
-			(is_https ? "https" : "http"), WC->http_host
+			 (is_https ? "https" : "http"), ChrPtr(WCC->http_host),
+			 (is_https ? "https" : "http"), ChrPtr(WCC->http_host)
 		);
 
 		serv_puts(buf);
@@ -382,78 +350,83 @@ void do_openid_login(void)
  */
 void finalize_openid_login(void)
 {
-	char buf[1024];
-	struct wcsession *WCC = WC;
+	StrBuf *Buf;
+	wcsession *WCC = WC;
 	int already_logged_in = (WCC->logged_in) ;
 	int linecount = 0;
-	char result[128] = "";
-	char username[128] = "";
-	char password[128] = "";
-	char logged_in_response[1024] = "";
-	char claimed_id[1024] = "";
+	StrBuf *result = NULL;
+	StrBuf *username = NULL;
+	StrBuf *password = NULL;
+	StrBuf *logged_in_response = NULL;
+	StrBuf *claimed_id = NULL;
 
 	if (havebstr("openid.mode")) {
 		if (!strcasecmp(bstr("openid.mode"), "id_res")) {
-
+			Buf = NewStrBuf();
 			serv_puts("OIDF");
-			serv_getln(buf, sizeof buf);
-
-			if (buf[0] == '8') {
+			StrBuf_ServGetln(Buf);
+			if (GetServerStatus(Buf, NULL) == 8) {
 				urlcontent *u;
 				void *U;
 				long HKLen;
-				char *HKey;
+				const char *HKey;
 				HashPos *Cursor;
 				
-				Cursor = GetNewHashPos ();
+				Cursor = GetNewHashPos (WCC->urlstrings, 0);
 				while (GetNextHashPos(WCC->urlstrings, Cursor, &HKLen, &HKey, &U)) {
 					u = (urlcontent*) U;
 					if (!strncasecmp(u->url_key, "openid.", 7)) {
-						serv_printf("%s|%s", &u->url_key[7], u->url_data);
+						serv_printf("%s|%s", &u->url_key[7], ChrPtr(u->url_data));
 					}
 				}
 
 				serv_puts("000");
 
 				linecount = 0;
-				while (serv_getln(buf, sizeof buf), strcmp(buf, "000")) {
-					if (linecount == 0) safestrncpy(result, buf, sizeof result);
-					if (!strcasecmp(result, "authenticate")) {
+				while (StrBuf_ServGetln(Buf), strcmp(ChrPtr(Buf), "000")) 
+				{
+					if (linecount == 0) result = NewStrBufDup(Buf);
+					if (!strcasecmp(ChrPtr(result), "authenticate")) {
 						if (linecount == 1) {
-							safestrncpy(username, buf, sizeof username);
+							username = NewStrBufDup(Buf);
 						}
 						else if (linecount == 2) {
-							safestrncpy(password, buf, sizeof password);
+							password = NewStrBufDup(Buf);
 						}
 						else if (linecount == 3) {
-							safestrncpy(logged_in_response, buf,
-								sizeof logged_in_response);
+							logged_in_response = NewStrBufDup(Buf);
 						}
 					}
-					else if (!strcasecmp(result, "verify_only")) {
+					else if (!strcasecmp(ChrPtr(result), "verify_only")) {
 						if (linecount == 1) {
-							safestrncpy(claimed_id, buf, sizeof claimed_id);
+							claimed_id = NewStrBufDup(Buf);
 						}
 						if (linecount == 2) {
-							safestrncpy(username, buf, sizeof username);
+							username = NewStrBufDup(Buf);
 						}
 					}
 					++linecount;
 				}
 			}
+			FreeStrBuf(&Buf);
 		}
 	}
 
 	/* If we were already logged in, this was an attempt to associate an OpenID account */
 	if (already_logged_in) {
 		display_openids();
+		FreeStrBuf(&result);
+		FreeStrBuf(&username);
+		FreeStrBuf(&password);
+		FreeStrBuf(&claimed_id);
+		FreeStrBuf(&logged_in_response);
 		return;
 	}
 
 	/* If this operation logged us in, either by connecting with an existing account or by
 	 * auto-creating one using Simple Registration Extension, we're already on our way.
 	 */
-	if (!strcasecmp(result, "authenticate")) {
+	if (!strcasecmp(ChrPtr(result), "authenticate")) {
 		become_logged_in(username, password, logged_in_response);
 	}
 
@@ -461,7 +434,7 @@ void finalize_openid_login(void)
 	 * or conflicts with an existing user.  Either way the user will need to specify a new name.
 	 */
 
-	else if (!strcasecmp(result, "verify_only")) {
+	else if (!strcasecmp(ChrPtr(result), "verify_only")) {
 		display_openid_name_request(claimed_id, username);
 	}
 
@@ -476,6 +449,11 @@ void finalize_openid_login(void)
 		display_openid_login(_("Your password was not accepted."));
 	}
 
+	FreeStrBuf(&result);
+	FreeStrBuf(&username);
+	FreeStrBuf(&password);
+	FreeStrBuf(&claimed_id);
+	FreeStrBuf(&logged_in_response);
 }
 
 
@@ -487,7 +465,7 @@ void finalize_openid_login(void)
  */
 void do_welcome(void)
 {
-	char buf[SIZ];
+	StrBuf *Buf;
 #ifdef XXX_NOT_FINISHED_YET_XXX
 	FILE *fp;
 	int i;
@@ -529,15 +507,17 @@ void do_welcome(void)
 	/*
 	 * Go to the user's preferred start page
 	 */
-	get_preference("startpage", buf, sizeof buf);
-	if (IsEmptyStr(buf)) {
-		safestrncpy(buf, "dotskip&room=_BASEROOM_", sizeof buf);
-		set_preference("startpage", buf, 1);
+	if (!get_preference("startpage", &Buf)) {
+		Buf = NewStrBuf ();
+		StrBufPrintf(Buf, "dotskip&room=_BASEROOM_");
+		set_preference("startpage", Buf, 1);
 	}
-	if (buf[0] == '/') {
-		strcpy(buf, &buf[1]);
+	if (ChrPtr(Buf)[0] == '/') {
+		StrBufCutLeft(Buf, 1);
 	}
-	http_redirect(buf);
+	if (StrLength(Buf) == 0)
+		StrBufAppendBufPlain(Buf, "dotgoto?room=_BASEROOM_", -1, 0);
+	http_redirect(ChrPtr(Buf));
 }
 
 
@@ -545,14 +525,7 @@ void do_welcome(void)
  * Disconnect from the Citadel server, and end this WebCit session
  */
 void end_webcit_session(void) {
-	char buf[256];
-
-	if (WC->logged_in) {
-		sprintf(buf, "%d", WC->current_iconbar);
-		set_preference("current_iconbar", buf, 0);
-		set_preference("floordiv_expanded", WC->floordiv_expanded, 1);
-	}
-
+	
 	serv_puts("QUIT");
 	WC->killthis = 1;
 	/* close() of citadel socket will be done by do_housekeeping() */
@@ -563,13 +536,18 @@ void end_webcit_session(void) {
  */
 void do_logout(void)
 {
+	wcsession *WCC = WC;
 	char buf[SIZ];
 
-	safestrncpy(WC->wc_username, "", sizeof WC->wc_username);
-	safestrncpy(WC->wc_password, "", sizeof WC->wc_password);
-	safestrncpy(WC->wc_roomname, "", sizeof WC->wc_roomname);
-	safestrncpy(WC->wc_fullname, "", sizeof WC->wc_fullname);
+	FlushStrBuf(WCC->wc_username);
+	FlushStrBuf(WCC->wc_password);
+	FlushStrBuf(WCC->wc_roomname);
+	FlushStrBuf(WCC->wc_fullname);
 
+	/* FIXME: this is to suppress the iconbar displaying, because we aren't
+	   actually logged out yet */
+	WCC->logged_in = 0;
+	
 	/** Calling output_headers() this way causes the cookies to be un-set */
 	output_headers(1, 1, 0, 1, 0, 0);
 
@@ -581,7 +559,7 @@ void do_logout(void)
 	serv_puts("MESG goodbye");
 	serv_getln(buf, sizeof buf);
 
-	if (WC->serv_sock >= 0) {
+	if (WCC->serv_sock >= 0) {
 		if (buf[0] == '1') {
 			fmout("CENTER");
 		} else {
@@ -636,7 +614,7 @@ void validate(void)
 			serv_printf("VALI %s|%s", buf, bstr("axlevel"));
 			serv_getln(buf, sizeof buf);
 			if (buf[0] != '2') {
-				wprintf("<b>%s</b><br />\n", &buf[4]);
+				wprintf("<b>%s</b><br>\n", &buf[4]);
 			}
 		}
 	}
@@ -647,19 +625,19 @@ void validate(void)
 	if (buf[0] == '2') {
 		wprintf("<b>");
 		wprintf(_("No users require validation at this time."));
-		wprintf("</b><br />\n");
+		wprintf("</b><br>\n");
 		wDumpContent(1);
 		return;
 	}
 	if (buf[0] != '3') {
-		wprintf("<b>%s</b><br />\n", &buf[4]);
+		wprintf("<b>%s</b><br>\n", &buf[4]);
 		wDumpContent(1);
 		return;
 	}
 
 	wprintf("<div class=\"fix_scrollbar_bug\">"
 		"<table class=\"auth_validate\"><tr><td>\n");
-	wprintf("<center>");
+	wprintf("<div id=\"validate\">");
 
 	safestrncpy(user, &buf[4], sizeof user);
 	serv_printf("GREG %s", user);
@@ -670,7 +648,7 @@ void validate(void)
 			serv_getln(buf, sizeof buf);
 			++a;
 			if (a == 1)
-				wprintf("#%s<br /><H1>%s</H1>",
+				wprintf("#%s<br><H1>%s</H1>",
 					buf, &cmd[4]);
 			if (a == 2) {
 				char *pch;
@@ -694,7 +672,7 @@ void validate(void)
 					count = 0;
 				switch (count){
 				case 0:
-					pch = _("verry weak");
+					pch = _("very weak");
 					break;
 				case 1:
 					pch = _("weak");
@@ -707,20 +685,20 @@ void validate(void)
 					pch = _("strong");
 				}
 
-				wprintf("PW: %s<br />\n", pch);
+				wprintf("PW: %s<br>\n", pch);
 			}
 			if (a == 3)
-				wprintf("%s<br />\n", buf);
+				wprintf("%s<br>\n", buf);
 			if (a == 4)
-				wprintf("%s<br />\n", buf);
+				wprintf("%s<br>\n", buf);
 			if (a == 5)
 				wprintf("%s, ", buf);
 			if (a == 6)
 				wprintf("%s ", buf);
 			if (a == 7)
-				wprintf("%s<br />\n", buf);
+				wprintf("%s<br>\n", buf);
 			if (a == 8)
-				wprintf("%s<br />\n", buf);
+				wprintf("%s<br>\n", buf);
 			if (a == 9)
 				wprintf(_("Current access level: %d (%s)\n"),
 					atoi(buf), axdefs[atoi(buf)]);
@@ -733,14 +711,14 @@ void validate(void)
 	wprintf(_("Select access level for this user:"));
 	wprintf("<br />\n");
 	for (a = 0; a <= 6; ++a) {
-		wprintf("<a href=\"validate?nonce=%ld?user=", WC->nonce);
+		wprintf("<a href=\"validate?nonce=%d?user=", WC->nonce);
 		urlescputs(user);
 		wprintf("&axlevel=%d\">%s</A>&nbsp;&nbsp;&nbsp;\n",
 			a, axdefs[a]);
 	}
 	wprintf("<br />\n");
 
-	wprintf("</CENTER>\n");
+	wprintf("</div>\n");
 	wprintf("</td></tr></table></div>\n");
 	wDumpContent(1);
 }
@@ -763,7 +741,7 @@ void display_reg(int during_login)
 		return;
 	}
 
-	vcard_msgnum = locate_user_vcard(WC->wc_fullname, -1);
+	vcard_msgnum = locate_user_vcard_in_this_room();
 	if (vcard_msgnum < 0L) {
 		if (during_login) do_welcome();
 		else display_main_menu();
@@ -787,12 +765,18 @@ void display_reg(int during_login)
  */
 void display_changepw(void)
 {
+	WCTemplputParams SubTP;
 	char buf[SIZ];
-
+	StrBuf *Buf;
 	output_headers(1, 1, 1, 0, 0, 0);
 
-	svput("BOXTITLE", WCS_STRING, _("Change your password"));
-	do_template("beginbox");
+	Buf = NewStrBufPlain(_("Change your password"), -1);
+	memset(&SubTP, 0, sizeof(WCTemplputParams));
+	SubTP.Filter.ContextType = CTX_STRBUF;
+	SubTP.Context = Buf;
+	DoTemplate(HKEY("beginbox"), NULL, &SubTP);
+
+	FreeStrBuf(&Buf);
 
 	if (!IsEmptyStr(WC->ImportantMessage)) {
 		wprintf("<span class=\"errormsg\">"
@@ -807,7 +791,7 @@ void display_changepw(void)
 	}
 
 	wprintf("<form name=\"changepwform\" action=\"changepw\" method=\"post\">\n");
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%ld\">\n", WC->nonce);
+	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
 	wprintf("<table class=\"altern\" ");
 	wprintf("<tr class=\"even\"><td>");
 	wprintf(_("Enter new password:"));
@@ -826,7 +810,7 @@ void display_changepw(void)
 	wprintf("</div>\n");
 	wprintf("</form>\n");
 
-	do_template("endbox");
+	do_template("endbox", NULL);
 	wDumpContent(1);
 }
 
@@ -870,7 +854,12 @@ void changepw(void)
 	serv_getln(buf, sizeof buf);
 	sprintf(WC->ImportantMessage, "%s", &buf[4]);
 	if (buf[0] == '2') {
-		safestrncpy(WC->wc_password, buf, sizeof WC->wc_password);
+		if (WC->wc_password == NULL)
+			WC->wc_password = NewStrBufPlain(buf, -1);
+		else {
+			FlushStrBuf(WC->wc_password);
+			StrBufAppendBufPlain(WC->wc_password,  buf, -1, 0);
+		}
 		display_main_menu();
 	}
 	else {
@@ -878,10 +867,52 @@ void changepw(void)
 	}
 }
 
-
-
-void InitModule_AUTH(void)
+int ConditionalAide(StrBuf *Target, WCTemplputParams *TP)
 {
-	WebcitAddUrlHandler(HKEY("do_welcome"), do_welcome, 0);
+	return (WC->is_aide == 0);
+}
+
+int ConditionalRoomAide(StrBuf *Target, WCTemplputParams *TP)
+{
+	return (WC->is_room_aide == 0);
+}
+
+int ConditionalIsLoggedIn(StrBuf *Target, WCTemplputParams *TP) {
+  return (WC->logged_in == 0);
+}
+int ConditionalRoomAcessDelete(StrBuf *Target, WCTemplputParams *TP)
+{
+	wcsession *WCC = WC;
+	return ( (WCC->is_room_aide) || (WCC->is_mailbox) || (WCC->room_flags2 & QR2_COLLABDEL) );
+}
+
+
+
+void _display_openid_login(void) {display_openid_login(NULL);}
+void _display_reg(void) {display_reg(0);}
+
+
+void 
+InitModule_AUTH
+(void)
+{
+	WebcitAddUrlHandler(HKEY("do_welcome"), do_welcome, ANONYMOUS);
+	WebcitAddUrlHandler(HKEY("login"), do_login, ANONYMOUS);
+	WebcitAddUrlHandler(HKEY("display_openid_login"), _display_openid_login, ANONYMOUS);
+	WebcitAddUrlHandler(HKEY("openid_login"), do_openid_login, ANONYMOUS);
+	WebcitAddUrlHandler(HKEY("finalize_openid_login"), finalize_openid_login, ANONYMOUS);
+	WebcitAddUrlHandler(HKEY("openid_manual_create"), openid_manual_create, ANONYMOUS);
+	WebcitAddUrlHandler(HKEY("do_logout"), do_logout, 0);
+	WebcitAddUrlHandler(HKEY("validate"), validate, 0);
+	WebcitAddUrlHandler(HKEY("display_reg"), _display_reg, 0);
+	WebcitAddUrlHandler(HKEY("display_changepw"), display_changepw, 0);
+	WebcitAddUrlHandler(HKEY("changepw"), changepw, 0);
+	WebcitAddUrlHandler(HKEY("termquit"), do_logout, 0);
+
+	RegisterConditional(HKEY("COND:AIDE"), 2, ConditionalAide, CTX_NONE);
+	RegisterConditional(HKEY("COND:ROOMAIDE"), 2, ConditionalRoomAide, CTX_NONE);
+	RegisterConditional(HKEY("COND:ACCESS:DELETE"), 2, ConditionalRoomAcessDelete, CTX_NONE);
+	RegisterConditional(HKEY("COND:LOGGEDIN"), 2, ConditionalIsLoggedIn, CTX_NONE);
+
 	return ;
 }
