@@ -1,5 +1,5 @@
 /*
- * $Id: autocompletion.c 6189 2008-04-11 14:26:13Z ajc $
+ * $Id: autocompletion.c 6463 2008-07-27 21:38:24Z dothebart $
  *
  * ajax-powered autocompletion...
  */
@@ -15,7 +15,7 @@ void recp_autocomplete(char *partial) {
 
 	output_headers(0, 0, 0, 0, 0, 0);
 
-	wprintf("Content-type: text/html\r\n"
+	hprintf("Content-type: text/html\r\n"
 		"Server: %s\r\n"
 		"Connection: close\r\n"
 		"Pragma: no-cache\r\n"
@@ -43,3 +43,18 @@ void recp_autocomplete(char *partial) {
 	wprintf("\r\n\r\n");
 	wDumpContent(0);
 }
+
+
+void _recp_autocomplete(void) {recp_autocomplete(bstr("recp"));}
+void _cc_autocomplete(void)   {recp_autocomplete(bstr("cc"));} 
+void _bcc_autocomplete(void)  {recp_autocomplete(bstr("bcc"));}
+
+
+void 
+InitModule_AUTO_COMPLETE
+(void)
+{
+	WebcitAddUrlHandler(HKEY("recp_autocomplete"), _recp_autocomplete, 0);
+	WebcitAddUrlHandler(HKEY("cc_autocomplete"),   _cc_autocomplete, 0);
+	WebcitAddUrlHandler(HKEY("bcc_autocomplete"),  _bcc_autocomplete, 0);
+}
