@@ -1,5 +1,5 @@
 /*
- * $Id: calendar.c 6255 2008-05-06 22:19:43Z dothebart $
+ * $Id: calendar.c 7477 2009-05-20 22:03:41Z dothebart $
  *
  * Functions which handle calendar objects and their processing/display.
  */
@@ -9,17 +9,18 @@
 
 
 /*
- * \brief Process a calendar object
- * ...at this point it's already been deserialized by cal_process_attachment()
- * \param cal the calendar object
- * \param recursion_level call stack depth ??????
- * \param msgnum number of the mesage in our db
- * \param cal_partnum of the calendar object ???? 
+ * Process a calendar object.  At this point it's already been deserialized by cal_process_attachment()
+ *
+ * cal:			the calendar object
+ * recursion_level:	Number of times we've recursed into this function
+ * msgnum:		Message number on the Citadel server
+ * cal_partnum:		MIME part number within that message containing the calendar object
  */
-void cal_process_object(icalcomponent *cal,
+void cal_process_object(StrBuf *Target,
+			icalcomponent *cal,
 			int recursion_level,
 			long msgnum,
-			char *cal_partnum) 
+			const char *cal_partnum) 
 {
 	icalcomponent *c;
 	icalproperty *method = NULL;
@@ -36,22 +37,31 @@ void cal_process_object(icalcomponent *cal,
 
 	sprintf(divname, "rsvp%04x", ++divcount);
 
-	/** Leading HTML for the display of this object */
-	if (recursion_level == 0) {
-		wprintf("<div class=\"mimepart\">\n");
+	/* Convert timezones to something easy to display.
+	 * It's safe to do this in memory because we're only changing it on the
+	 * display side -- when we tell the server to do something with the object,
+	 * the server will be working with its original copy in the database.
+	 */
+	if ((cal) && (recursion_level == 0)) {
+		ical_dezonify(cal);
 	}
 
-	/** Look for a method */
+	/* Leading HTML for the display of this object */
+	if (recursion_level == 0) {
+		StrBufAppendPrintf(Target, "<div class=\"mimepart\">\n");
+	}
+
+	/* Look for a method */
 	method = icalcomponent_get_first_property(cal, ICAL_METHOD_PROPERTY);
 
-	/** See what we need to do with this */
+	/* See what we need to do with this */
 	if (method != NULL) {
-		the_method = icalproperty_get_method(method);
 		char *title;
+		the_method = icalproperty_get_method(method);
 
-		wprintf("<div id=\"%s_title\">", divname);
-		wprintf("<img src=\"static/calarea_48x.gif\">");
-		wprintf("<span>");
+		StrBufAppendPrintf(Target, "<div id=\"%s_title\">", divname);
+		StrBufAppendPrintf(Target, "<img src=\"static/calarea_48x.gif\">");
+		StrBufAppendPrintf(Target, "<span>");
 		switch(the_method) {
 		case ICAL_METHOD_REQUEST:
 			title = _("Meeting invitation");
@@ -66,39 +76,38 @@ void cal_process_object(icalcomponent *cal,
 			title = _("This is an unknown type of calendar item.");
 			break;
 		}
-		wprintf("</span>");
+		StrBufAppendPrintf(Target, "</span>");
 
-		wprintf("&nbsp;&nbsp;%s",title);
-		wprintf("</div>");
+		StrBufAppendPrintf(Target, "&nbsp;&nbsp;%s",title);
+		StrBufAppendPrintf(Target, "</div>");
 	}
 
-	wprintf("<dl>");
+	StrBufAppendPrintf(Target, "<dl>");
       	p = icalcomponent_get_first_property(cal, ICAL_SUMMARY_PROPERTY);
 	if (p != NULL) {
-		wprintf("<dt>");
-		wprintf(_("Summary:"));
-		wprintf("</dt><dd>");
-		escputs((char *)icalproperty_get_comment(p));
-		wprintf("</dd>\n");
+		StrBufAppendPrintf(Target, "<dt>");
+		StrBufAppendPrintf(Target, _("Summary:"));
+		StrBufAppendPrintf(Target, "</dt><dd>");
+		StrEscAppend(Target, NULL, (char *)icalproperty_get_comment(p), 0, 0);
+		StrBufAppendPrintf(Target, "</dd>\n");
 	}
 
       	p = icalcomponent_get_first_property(cal, ICAL_LOCATION_PROPERTY);
 	if (p != NULL) {
-		wprintf("<dt>");
-		wprintf(_("Location:"));
-		wprintf("</dt><dd>");
-		escputs((char *)icalproperty_get_comment(p));
-		wprintf("</dd>\n");
+		StrBufAppendPrintf(Target, "<dt>");
+		StrBufAppendPrintf(Target, _("Location:"));
+		StrBufAppendPrintf(Target, "</dt><dd>");
+		StrEscAppend(Target, NULL, (char *)icalproperty_get_comment(p), 0, 0);
+		StrBufAppendPrintf(Target, "</dd>\n");
 	}
 
-	/**
+	/*
 	 * Only show start/end times if we're actually looking at the VEVENT
 	 * component.  Otherwise it shows bogus dates for things like timezone.
 	 */
 	if (icalcomponent_isa(cal) == ICAL_VEVENT_COMPONENT) {
 
-      		p = icalcomponent_get_first_property(cal,
-						     ICAL_DTSTART_PROPERTY);
+      		p = icalcomponent_get_first_property(cal, ICAL_DTSTART_PROPERTY);
 		if (p != NULL) {
 			t = icalproperty_get_dtstart(p);
 
@@ -110,16 +119,16 @@ void cal_process_object(icalcomponent *cal,
 				d_tm.tm_mon = t.month - 1;
 				d_tm.tm_mday = t.day;
 				wc_strftime(d_str, sizeof d_str, "%x", &d_tm);
-				wprintf("<dt>");
-				wprintf(_("Date:"));
-				wprintf("</dt><dd>%s</dd>", d_str);
+				StrBufAppendPrintf(Target, "<dt>");
+				StrBufAppendPrintf(Target, _("Date:"));
+				StrBufAppendPrintf(Target, "</dt><dd>%s</dd>", d_str);
 			}
 			else {
 				tt = icaltime_as_timet(t);
-				webcit_fmt_date(buf, tt, 0);
-				wprintf("<dt>");
-				wprintf(_("Starting date/time:"));
-				wprintf("</dt><dd>%s</dd>", buf);
+				webcit_fmt_date(buf, 256, tt, DATEFMT_FULL);
+				StrBufAppendPrintf(Target, "<dt>");
+				StrBufAppendPrintf(Target, _("Starting date/time:"));
+				StrBufAppendPrintf(Target, "</dt><dd>%s</dd>", buf);
 			}
 		}
 	
@@ -127,55 +136,63 @@ void cal_process_object(icalcomponent *cal,
 		if (p != NULL) {
 			t = icalproperty_get_dtend(p);
 			tt = icaltime_as_timet(t);
-			webcit_fmt_date(buf, tt, 0);
-			wprintf("<dt>");
-			wprintf(_("Ending date/time:"));
-			wprintf("</dt><dd>%s</dd>", buf);
+			webcit_fmt_date(buf, 256, tt, DATEFMT_FULL);
+			StrBufAppendPrintf(Target, "<dt>");
+			StrBufAppendPrintf(Target, _("Ending date/time:"));
+			StrBufAppendPrintf(Target, "</dt><dd>%s</dd>", buf);
 		}
 
 	}
 
       	p = icalcomponent_get_first_property(cal, ICAL_DESCRIPTION_PROPERTY);
 	if (p != NULL) {
-		wprintf("<dt>");
-		wprintf(_("Description:"));
-		wprintf("</dt><dd>");
-		escputs((char *)icalproperty_get_comment(p));
-		wprintf("</dd>\n");
+		StrBufAppendPrintf(Target, "<dt>");
+		StrBufAppendPrintf(Target, _("Description:"));
+		StrBufAppendPrintf(Target, "</dt><dd>");
+		StrEscAppend(Target, NULL, (char *)icalproperty_get_comment(p), 0, 0);
+		StrBufAppendPrintf(Target, "</dd>\n");
 	}
 
-	/** If the component has attendees, iterate through them. */
+	if (icalcomponent_get_first_property(cal, ICAL_RRULE_PROPERTY)) {
+		/* Unusual string syntax used here in order to re-use existing translations */
+		StrBufAppendPrintf(Target, "<dt>%s:</dt><dd>%s.</dd>\n",
+			_("Recurrence"),
+			_("This is a recurring event")
+		);
+	}
+
+	/* If the component has attendees, iterate through them. */
 	for (p = icalcomponent_get_first_property(cal, ICAL_ATTENDEE_PROPERTY); 
 	     (p != NULL); 
 	     p = icalcomponent_get_next_property(cal, ICAL_ATTENDEE_PROPERTY)) {
-		wprintf("<dt>");
-		wprintf(_("Attendee:"));
-		wprintf("</dt><dd>");
+		StrBufAppendPrintf(Target, "<dt>");
+		StrBufAppendPrintf(Target, _("Attendee:"));
+		StrBufAppendPrintf(Target, "</dt><dd>");
 		safestrncpy(buf, icalproperty_get_attendee(p), sizeof buf);
 		if (!strncasecmp(buf, "MAILTO:", 7)) {
 
 			/** screen name or email address */
 			strcpy(buf, &buf[7]);
 			striplt(buf);
-			escputs(buf);
-			wprintf(" ");
+			StrEscAppend(Target, NULL, buf, 0, 0);
+			StrBufAppendPrintf(Target, " ");
 
 			/** participant status */
 			partstat_as_string(buf, p);
-			escputs(buf);
+			StrEscAppend(Target, NULL, buf, 0, 0);
 		}
-		wprintf("</dd>\n");
+		StrBufAppendPrintf(Target, "</dd>\n");
 	}
 
-	/** If the component has subcomponents, recurse through them. */
+	/* If the component has subcomponents, recurse through them. */
 	for (c = icalcomponent_get_first_component(cal, ICAL_ANY_COMPONENT);
 	     (c != 0);
 	     c = icalcomponent_get_next_component(cal, ICAL_ANY_COMPONENT)) {
 		/* Recursively process subcomponent */
-		cal_process_object(c, recursion_level+1, msgnum, cal_partnum);
+		cal_process_object(Target, c, recursion_level+1, msgnum, cal_partnum);
 	}
 
-	/** If this is a REQUEST, display conflicts and buttons */
+	/* If this is a REQUEST, display conflicts and buttons */
 	if (the_method == ICAL_METHOD_REQUEST) {
 
 		/* Check for conflicts */
@@ -196,23 +213,23 @@ void cal_process_object(icalcomponent *cal,
 						 _("This event would conflict with '%s' which is already in your calendar."), conflict_name);
 				}
 
-				wprintf("<dt>%s",
+				StrBufAppendPrintf(Target, "<dt>%s",
 					(is_update ?
 					 _("Update:") :
 					 _("CONFLICT:")
 						)
 					);
-				wprintf("</dt><dd>");
-				escputs(conflict_message);
-				wprintf("</dd>\n");
+				StrBufAppendPrintf(Target, "</dt><dd>");
+				StrEscAppend(Target, NULL, conflict_message, 0, 0);
+				StrBufAppendPrintf(Target, "</dd>\n");
 			}
 		}
 		lprintf(9, "...done.\n");
 
-		wprintf("</dl>");
+		StrBufAppendPrintf(Target, "</dl>");
 
-		/** Display the Accept/Decline buttons */
-		wprintf("<p id=\"%s_question\">"
+		/* Display the Accept/Decline buttons */
+		StrBufAppendPrintf(Target, "<p id=\"%s_question\">"
 			"%s "
 			"&nbsp;&nbsp;&nbsp;<span class=\"button_link\"> "
 			"<a href=\"javascript:RespondToInvitation('%s_question','%s_title','%ld','%s','Accept');\">%s</a>"
@@ -230,18 +247,11 @@ void cal_process_object(icalcomponent *cal,
 
 	}
 
-	/** If this is a REPLY, display update button */
+	/* If this is a REPLY, display update button */
 	if (the_method == ICAL_METHOD_REPLY) {
 
-		/** \todo  In the future, if we want to validate this object before \
-		 * continuing, we can do it this way:
-		 serv_printf("ICAL whatever|%ld|%s|", msgnum, cal_partnum);
-		 serv_getln(buf, sizeof buf);
-		 }
-		***********/
-
-		/** Display the update buttons */
-		wprintf("<p id=\"%s_question\" >"
+		/* Display the update buttons */
+		StrBufAppendPrintf(Target, "<p id=\"%s_question\" >"
 			"%s "
 			"&nbsp;&nbsp;&nbsp;<span class=\"button_link\"> "
 			"<a href=\"javascript:HandleRSVP('%s_question','%s_title','%ld','%s','Update');\">%s</a>"
@@ -256,35 +266,30 @@ void cal_process_object(icalcomponent *cal,
 	
 	}
 	
-	/** Trailing HTML for the display of this object */
+	/* Trailing HTML for the display of this object */
 	if (recursion_level == 0) {
-		wprintf("<p>&nbsp;</p></div>\n");
+		StrBufAppendPrintf(Target, "<p>&nbsp;</p></div>\n");
 	}
 }
 
 
-/**
- * \brief process calendar mail atachment
- * Deserialize a calendar object in a message so it can be processed.
- * (This is the main entry point for these things)
- * \param part_source the part of the message we want to parse
- * \param msgnum number of the mesage in our db
- * \param cal_partnum the number of the calendar item
+/*
+ * Deserialize a calendar object in a message so it can be displayed.
+ *
  */
-void cal_process_attachment(char *part_source, long msgnum, char *cal_partnum) 
+void cal_process_attachment(wc_mime_attachment *Mime) 
 {
 	icalcomponent *cal;
 
-	cal = icalcomponent_new_from_string(part_source);
-
+	cal = icalcomponent_new_from_string(ChrPtr(Mime->Data));
+	FlushStrBuf(Mime->Data);
 	if (cal == NULL) {
-		wprintf(_("There was an error parsing this calendar item."));
-		wprintf("<br />\n");
+		StrBufAppendPrintf(Mime->Data, _("There was an error parsing this calendar item."));
+		StrBufAppendPrintf(Mime->Data, "<br />\n");
 		return;
 	}
 
-	ical_dezonify(cal);
-	cal_process_object(cal, 0, msgnum, cal_partnum);
+	cal_process_object(Mime->Data, cal, 0, Mime->msgnum, ChrPtr(Mime->PartNum));
 
 	/* Free the memory we obtained from libical's constructor */
 	icalcomponent_free(cal);
@@ -375,30 +380,9 @@ void handle_rsvp(void)
 
 
 
-/*@}*/
-/*-----------------------------------------------------------------------**/
 
-
-
-/**
- * \defgroup MsgDisplayHandlers Display handlers for message reading 
- * \ingroup Calendaring
- */
-
-/*@{*/
-
-int Flathash(const char *str, long len)
-{
-	if (len != sizeof (int))
-		return 0;
-	else return *(int*)str;
-}
-
-
-
-/**
- * \brief clean up ical memory
- * todo this could get trouble with future ical versions 
+/*
+ * free memory allocated using libical
  */
 void delete_cal(void *vCal)
 {
@@ -408,62 +392,200 @@ void delete_cal(void *vCal)
 	free(Cal);
 }
 
-/**
- * \brief get items, keep them.
- * If we're reading calendar items, just store them for now.  We have to
- * sort and re-output them later when we draw the calendar.
- * \param cal Our calendar to process
- * \param msgnum number of the mesage in our db
+/*
+ * This is the meat-and-bones of the first part of our two-phase calendar display.
+ * As we encounter calendar items in messages being read from the server, we break out
+ * any iCalendar objects and store them in a hash table.  Later on, the second phase will
+ * use this hash table to render the calendar for display.
  */
-void display_individual_cal(icalcomponent *cal, long msgnum, char *from, int unread)
+void display_individual_cal(icalcomponent *cal, long msgnum, char *from, int unread, struct calview *calv)
 {
 	icalproperty *ps = NULL;
-	struct icaltimetype t;
-	struct wcsession *WCC = WC;
+	struct icaltimetype dtstart, dtend;
+	struct icaldurationtype dur;
+	wcsession *WCC = WC;
 	disp_cal *Cal;
 	size_t len;
+	time_t final_recurrence = 0;
+	icalcomponent *cptr = NULL;
+
+	/* recur variables */
+	icalproperty *rrule = NULL;
+	struct icalrecurrencetype recur;
+	icalrecur_iterator *ritr = NULL;
+	struct icaltimetype next;
+	int num_recur = 0;
+	int stop_rr = 0;
+
+	dtstart = icaltime_null_time();
+	dtend = icaltime_null_time();
 	
 	if (WCC->disp_cal_items == NULL)
 		WCC->disp_cal_items = NewHash(0, Flathash);
-	
+
+	/* Note: anything we do here, we also have to do below for the recurrences. */
 	Cal = (disp_cal*) malloc(sizeof(disp_cal));
 	memset(Cal, 0, sizeof(disp_cal));
-
 	Cal->cal = icalcomponent_new_clone(cal);
+
+	/* Dezonify and decapsulate at the very last moment */
+	/* lprintf(9, "INITIAL: %s\n", icaltime_as_ical_string(icalproperty_get_dtstart(
+		icalcomponent_get_first_property(icalcomponent_get_first_component(
+		Cal->cal, ICAL_VEVENT_COMPONENT), ICAL_DTSTART_PROPERTY)))
+	); */
+	ical_dezonify(Cal->cal);
+	if (icalcomponent_isa(Cal->cal) != ICAL_VEVENT_COMPONENT) {
+		cptr = icalcomponent_get_first_component(Cal->cal, ICAL_VEVENT_COMPONENT);
+		if (cptr) {
+			cptr = icalcomponent_new_clone(cptr);
+			icalcomponent_free(Cal->cal);
+			Cal->cal = cptr;
+		}
+	}
+
 	Cal->unread = unread;
 	len = strlen(from);
 	Cal->from = (char*)malloc(len+ 1);
 	memcpy(Cal->from, from, len + 1);
-	ical_dezonify(Cal->cal);
 	Cal->cal_msgnum = msgnum;
 
-	//! Precalculate some Values we can use for easy comparison later.
+	/* Precalculate the starting date and time of this event, and store it in our top-level
+	 * structure.  Later, when we are rendering the calendar, we can just peek at these values
+	 * without having to break apart every calendar item.
+	 */
 	ps = icalcomponent_get_first_property(Cal->cal, ICAL_DTSTART_PROPERTY);
 	if (ps != NULL) {
-		t = icalproperty_get_dtstart(ps);
-		Cal->event_start = icaltime_as_timet(t);
+		dtstart = icalproperty_get_dtstart(ps);
+		Cal->event_start = icaltime_as_timet(dtstart);
 	}
+
+	/* Do the same for the ending date and time.  It makes the day view much easier to render. */
 	ps = icalcomponent_get_first_property(Cal->cal, ICAL_DTEND_PROPERTY);
-	if (ps != NULL) { //!  Precalc the end day and end day + hour
-		t = icalproperty_get_dtstart(ps);
-		Cal->event_end = icaltime_as_timet(t);
+	if (ps != NULL) {
+		dtend = icalproperty_get_dtend(ps);
+		Cal->event_end = icaltime_as_timet(dtend);
 	}
+
+	/* Store it in the hash list. */
 	Put(WCC->disp_cal_items, 
 	    (char*) &Cal->event_start,
 	    sizeof(Cal->event_start), 
 	    Cal, 
 	    delete_cal);
+
+	/****************************** handle recurring events ******************************/
+
+	if (icaltime_is_null_time(dtstart)) return;	/* Can't recur without a start time */
+
+	if (!icaltime_is_null_time(dtend)) {		/* Need duration for recurrences */
+		dur = icaltime_subtract(dtend, dtstart);
+	}
+
+	/*
+	 * Just let libical iterate the recurrence, and keep looping back to the top of this function,
+	 * adding new hash entries that all point back to the same msgnum, until either the iteration
+	 * stops or some outer bound is reached.  The display code will automatically do the Right Thing.
+	 */
+	cptr = cal;
+	if (icalcomponent_isa(cptr) != ICAL_VEVENT_COMPONENT) {
+		cptr = icalcomponent_get_first_component(cptr, ICAL_VEVENT_COMPONENT);
+	}
+	if (!cptr) return;
+	ps = icalcomponent_get_first_property(cptr, ICAL_DTSTART_PROPERTY);
+	if (ps == NULL) return;
+	dtstart = icalproperty_get_dtstart(ps);
+	rrule = icalcomponent_get_first_property(cptr, ICAL_RRULE_PROPERTY);
+	if (!rrule) return;
+	recur = icalproperty_get_rrule(rrule);
+	ritr = icalrecur_iterator_new(recur, dtstart);
+	if (!ritr) return;
+
+	while (next = icalrecur_iterator_next(ritr), ((!icaltime_is_null_time(next))&&(!stop_rr)) ) {
+		++num_recur;
+		if (num_recur > 1) {		/* Skip the first one.  We already did it at the root. */
+			icalcomponent *cptr;
+			/* lprintf(9, "REPEATS: %s\n", icaltime_as_ical_string(next)); */
+
+			/* Note: anything we do here, we also have to do above for the root event. */
+			Cal = (disp_cal*) malloc(sizeof(disp_cal));
+			memset(Cal, 0, sizeof(disp_cal));
+			Cal->cal = icalcomponent_new_clone(cal);
+			Cal->unread = unread;
+			len = strlen(from);
+			Cal->from = (char*)malloc(len+ 1);
+			memcpy(Cal->from, from, len + 1);
+			Cal->cal_msgnum = msgnum;
+
+			if (icalcomponent_isa(Cal->cal) == ICAL_VEVENT_COMPONENT) {
+				cptr = Cal->cal;
+			}
+			else {
+				cptr = icalcomponent_get_first_component(Cal->cal, ICAL_VEVENT_COMPONENT);
+			}
+			if (cptr) {
+				ps = icalcomponent_get_first_property(cptr, ICAL_DTSTART_PROPERTY);
+				if (ps != NULL) {
+					icalcomponent_remove_property(cptr, ps);
+					ps = icalproperty_new_dtstart(next);
+					icalcomponent_add_property(cptr, ps);
+	
+					Cal->event_start = icaltime_as_timet(next);
+					final_recurrence = Cal->event_start;
+				}
+
+				ps = icalcomponent_get_first_property(cptr, ICAL_DTEND_PROPERTY);
+				if (ps != NULL) {
+					icalcomponent_remove_property(cptr, ps);
+	
+					/* Make a new dtend */
+					ps = icalproperty_new_dtend(icaltime_add(next, dur));
+		
+					/* and stick it somewhere */
+					icalcomponent_add_property(cptr, ps);
+				}
+
+			}
+
+			/* Dezonify and decapsulate at the very last moment */
+			ical_dezonify(Cal->cal);
+			if (icalcomponent_isa(Cal->cal) != ICAL_VEVENT_COMPONENT) {
+				cptr = icalcomponent_get_first_component(Cal->cal, ICAL_VEVENT_COMPONENT);
+				if (cptr) {
+					cptr = icalcomponent_new_clone(cptr);
+					icalcomponent_free(Cal->cal);
+					Cal->cal = cptr;
+				}
+			}
+
+			if ( (Cal->event_start > calv->lower_bound)
+			   && (Cal->event_start < calv->upper_bound) ) {
+				Put(WCC->disp_cal_items, 
+					(char*) &Cal->event_start,
+					sizeof(Cal->event_start), 
+					Cal, 
+					delete_cal
+				);
+			}
+			else {
+				delete_cal(Cal);
+			}
+
+			/* If an upper bound is set, stop when we go out of scope */
+			if (final_recurrence > calv->upper_bound) stop_rr = 1;
+		}
+	}
+	icalrecur_iterator_free(ritr);
+	/* lprintf(9, "Performed %d recurrences; final one is %s", num_recur, ctime(&final_recurrence)); */
+
 }
 
 
 
-/**
- * \brief edit a task
+/*
  * Display a task by itself (for editing)
- * \param supplied_vtodo the todo item we want to edit
- * \param msgnum number of the mesage in our db
  */
-void display_edit_individual_task(icalcomponent *supplied_vtodo, long msgnum, char *from, int unread) 
+void display_edit_individual_task(icalcomponent *supplied_vtodo, long msgnum, char *from,
+			int unread, struct calview *calv)
 {
 	icalcomponent *vtodo;
 	icalproperty *p;
@@ -477,7 +599,12 @@ void display_edit_individual_task(icalcomponent *supplied_vtodo, long msgnum, ch
 	if (supplied_vtodo != NULL) {
 		vtodo = supplied_vtodo;
 
-		/**
+		/*
+		 * It's safe to convert to UTC here because there are no recurrences to worry about.
+		 */
+		ical_dezonify(vtodo);
+
+		/*
 		 * If we're looking at a fully encapsulated VCALENDAR
 		 * rather than a VTODO component, attempt to use the first
 		 * relevant VTODO subcomponent.  If there is none, the
@@ -490,8 +617,7 @@ void display_edit_individual_task(icalcomponent *supplied_vtodo, long msgnum, ch
 				icalcomponent_get_first_component(
 					vtodo, ICAL_VTODO_COMPONENT
 					), 
-				msgnum,
-				from, unread
+				msgnum, from, unread, calv
 				);
 			return;
 		}
@@ -501,11 +627,11 @@ void display_edit_individual_task(icalcomponent *supplied_vtodo, long msgnum, ch
 		created_new_vtodo = 1;
 	}
 	
-	// TODO: Can we take all this and move it into a template?	
+	/*/ TODO: Can we take all this and move it into a template?	 */
 	output_headers(1, 1, 1, 0, 0, 0);
 	wprintf("<!-- start task edit form -->");
 	p = icalcomponent_get_first_property(vtodo, ICAL_SUMMARY_PROPERTY);
-	// Get summary early for title
+	/* Get summary early for title */
 	wprintf("<div class=\"box\">\n");
 	wprintf("<div class=\"boxlabel\">");
 	wprintf(_("Edit task"));
@@ -518,9 +644,10 @@ void display_edit_individual_task(icalcomponent *supplied_vtodo, long msgnum, ch
 	wprintf("<div class=\"boxcontent\">\n");
 	wprintf("<FORM METHOD=\"POST\" action=\"save_task\">\n");
 	wprintf("<div style=\"display: none;\">\n	");
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%ld\">\n", WC->nonce);
-	wprintf("<INPUT TYPE=\"hidden\" NAME=\"msgnum\" VALUE=\"%ld\">\n",
-		msgnum);
+	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
+	wprintf("<INPUT TYPE=\"hidden\" NAME=\"msgnum\" VALUE=\"%ld\">\n", msgnum);
+	wprintf("<INPUT TYPE=\"hidden\" NAME=\"return_to_summary\" VALUE=\"%d\">\n",
+		ibstr("return_to_summary"));
 	wprintf("</div>");
 	wprintf("<table class=\"calendar_background\"><tr><td>");
 	wprintf("<TABLE STYLE=\"border: none;\">\n");
@@ -548,6 +675,7 @@ void display_edit_individual_task(icalcomponent *supplied_vtodo, long msgnum, ch
 	wprintf(_("No date"));
 	
 	wprintf(" ");
+	wprintf("<span ID=\"dtstart_date\">");
 	wprintf(_("or"));
 	wprintf(" ");
 	if (p != NULL) {
@@ -555,8 +683,15 @@ void display_edit_individual_task(icalcomponent *supplied_vtodo, long msgnum, ch
 	}
 	else
 		IcalTime = icaltime_current_time_with_zone(get_default_icaltimezone());
-	display_icaltimetype_as_webform(&IcalTime, "dtstart");
-	wprintf("</TD></TR>\n");
+	display_icaltimetype_as_webform(&IcalTime, "dtstart", 0);
+
+	wprintf("<INPUT TYPE=\"CHECKBOX\" NAME=\"dtstart_time_assoc\" ID=\"dtstart_time_assoc\" VALUE=\"yes\"");
+	if (!IcalTime.is_date) {
+		wprintf("CHECKED=\"CHECKED\"");
+	}
+	wprintf(">");
+	wprintf(_("Time associated"));
+	wprintf("</span></TD></TR>\n");
 
 	wprintf("<TR><TD>");
 	wprintf(_("Due date:"));
@@ -569,6 +704,7 @@ void display_edit_individual_task(icalcomponent *supplied_vtodo, long msgnum, ch
 	wprintf(">");
 	wprintf(_("No date"));
 	wprintf(" ");
+	wprintf("<span ID=\"due_date\">\n");
 	wprintf(_("or"));
 	wprintf(" ");
 	if (p != NULL) {
@@ -576,20 +712,26 @@ void display_edit_individual_task(icalcomponent *supplied_vtodo, long msgnum, ch
 	}
 	else
 		IcalTime = icaltime_current_time_with_zone(get_default_icaltimezone());
-	display_icaltimetype_as_webform(&IcalTime, "due");
-		
-	wprintf("</TD></TR>\n");
+	display_icaltimetype_as_webform(&IcalTime, "due", 0);
+
+	wprintf("<INPUT TYPE=\"CHECKBOX\" NAME=\"due_time_assoc\" ID=\"due_time_assoc\" VALUE=\"yes\"");
+	if (!IcalTime.is_date) {
+		wprintf("CHECKED=\"CHECKED\"");
+	}
+	wprintf(">");
+	wprintf(_("Time associated"));
+	wprintf("</span></TD></TR>\n");
 	todoStatus = icalcomponent_get_status(vtodo);
 	wprintf("<TR><TD>\n");
 	wprintf(_("Completed:"));
 	wprintf("</TD><TD>");
-	wprintf("<INPUT TYPE=\"CHECKBOX\" NAME=\"STATUS\" VALUE=\"COMPLETED\"");
+	wprintf("<INPUT TYPE=\"CHECKBOX\" NAME=\"status\" VALUE=\"COMPLETED\"");
 	if (todoStatus == ICAL_STATUS_COMPLETED) {
 		wprintf(" CHECKED=\"CHECKED\"");
 	} 
 	wprintf(" >");
 	wprintf("</TD></TR>");
-	// start category field
+	/* start category field */
 	p = icalcomponent_get_first_property(vtodo, ICAL_CATEGORIES_PROPERTY);
 	wprintf("<TR><TD>");
 	wprintf(_("Category:"));
@@ -600,7 +742,7 @@ void display_edit_individual_task(icalcomponent *supplied_vtodo, long msgnum, ch
 	}
 	wprintf("\">");
 	wprintf("</TD></TR>\n	");
-	// end category field
+	/* end category field */
 	wprintf("<TR><TD>");
 	wprintf(_("Description:"));
 	wprintf("</TD><TD>");
@@ -640,7 +782,8 @@ void display_edit_individual_task(icalcomponent *supplied_vtodo, long msgnum, ch
  * \param supplied_vtodo the task to save
  * \param msgnum number of the mesage in our db
  */
-void save_individual_task(icalcomponent *supplied_vtodo, long msgnum, char* from, int unread) 
+void save_individual_task(icalcomponent *supplied_vtodo, long msgnum, char* from, int unread,
+				struct calview *calv)
 {
 	char buf[SIZ];
 	int delete_existing = 0;
@@ -665,7 +808,7 @@ void save_individual_task(icalcomponent *supplied_vtodo, long msgnum, char* from
 			save_individual_task(
 				icalcomponent_get_first_component(
 					vtodo, ICAL_VTODO_COMPONENT), 
-				msgnum, from, unread
+				msgnum, from, unread, calv
 				);
 			return;
 		}
@@ -690,7 +833,7 @@ void save_individual_task(icalcomponent *supplied_vtodo, long msgnum, char* from
 						   icalproperty_new_summary(bstr("summary")));
 		} else {
 			icalcomponent_add_property(vtodo,
-						   icalproperty_new_summary("Untitled Task"));
+						   icalproperty_new_summary(_("Untitled Task")));
 		}
 	
 		while (prop = icalcomponent_get_first_property(vtodo,
@@ -698,7 +841,7 @@ void save_individual_task(icalcomponent *supplied_vtodo, long msgnum, char* from
 			icalcomponent_remove_property(vtodo, prop);
 			icalproperty_free(prop);
 		}
-		if (!IsEmptyStr(bstr("description"))) {
+		if (havebstr("description")) {
 			icalcomponent_add_property(vtodo,
 						   icalproperty_new_description(bstr("description")));
 		}
@@ -709,7 +852,12 @@ void save_individual_task(icalcomponent *supplied_vtodo, long msgnum, char* from
 			icalproperty_free(prop);
 		}
 		if (IsEmptyStr(bstr("nodtstart"))) {
-			icaltime_from_webform(&t, "dtstart");
+			if (yesbstr("dtstart_time")) {
+				icaltime_from_webform(&t, "dtstart");
+			}
+			else {
+				icaltime_from_webform_dateonly(&t, "dtstart");
+			}
 			icalcomponent_add_property(vtodo,
 						   icalproperty_new_dtstart(t)
 				);
@@ -719,10 +867,23 @@ void save_individual_task(icalcomponent *supplied_vtodo, long msgnum, char* from
 			icalcomponent_remove_property(vtodo,prop);
 			icalproperty_free(prop);
 		}
-		if (!IsEmptyStr(bstr("status"))) {
-			icalproperty_status taskStatus = icalproperty_string_to_status(
-				bstr("status"));
+		while(prop = icalcomponent_get_first_property(vtodo,
+							      ICAL_PERCENTCOMPLETE_PROPERTY), prop != NULL) {
+			icalcomponent_remove_property(vtodo,prop);
+			icalproperty_free(prop);
+		}
+
+		if (havebstr("status")) {
+			icalproperty_status taskStatus = icalproperty_string_to_status(bstr("status"));
 			icalcomponent_set_status(vtodo, taskStatus);
+			icalcomponent_add_property(vtodo,
+				icalproperty_new_percentcomplete(
+					(strcasecmp(bstr("status"), "completed") ? 0 : 100)
+				)
+			);
+		}
+		else {
+			icalcomponent_add_property(vtodo, icalproperty_new_percentcomplete(0));
 		}
 		while (prop = icalcomponent_get_first_property(vtodo,
 							       ICAL_CATEGORIES_PROPERTY), prop != NULL) {
@@ -739,7 +900,12 @@ void save_individual_task(icalcomponent *supplied_vtodo, long msgnum, char* from
 			icalproperty_free(prop);
 		}
 		if (IsEmptyStr(bstr("nodue"))) {
-			icaltime_from_webform(&t, "due");
+			if (yesbstr("due_time")) {
+				icaltime_from_webform(&t, "due");
+			}
+			else {
+				icaltime_from_webform_dateonly(&t, "due");
+			}
 			icalcomponent_add_property(vtodo,
 						   icalproperty_new_due(t)
 				);
@@ -815,27 +981,27 @@ void save_individual_task(icalcomponent *supplied_vtodo, long msgnum, char* from
 		icalcomponent_free(vtodo);
 	}
 
-	/** Go back to the task list */
-	readloop("readfwd");
+	/* Go back to wherever we came from */
+	if (ibstr("return_to_summary") == 1) {
+		summary();
+	}
+	else {
+		readloop(readfwd);
+	}
 }
 
 
 
-/**
- * \brief generic item handler
- * Code common to all display handlers.  Given a message number and a MIME
+/*
+ * Code common to all icalendar display handlers.  Given a message number and a MIME
  * type, we load the message and hunt for that MIME type.  If found, we load
  * the relevant part, deserialize it into a libical component, filter it for
  * the requested object type, and feed it to the specified handler.
- * \param mimetype mimetyp of our object
- * \param which_kind sort of ical type
- * \param msgnum number of the mesage in our db
- * \param callback a funcion \todo
- *
  */
-void display_using_handler(long msgnum, int unread,
+void load_ical_object(long msgnum, int unread,
 			   icalcomponent_kind which_kind,
-			   void (*callback)(icalcomponent *, long, char*, int)
+			   void (*callback)(icalcomponent *, long, char*, int, struct calview *),
+			   struct calview *calv
 	) 
 {
 	char buf[1024];
@@ -866,6 +1032,7 @@ void display_using_handler(long msgnum, int unread,
 			if (  (!strcasecmp(mime_content_type, "text/calendar"))
 			      || (!strcasecmp(mime_content_type, "application/ics"))
 			      || (!strcasecmp(mime_content_type, "text/vtodo"))
+                              || (!strcasecmp(mime_content_type, "text/todo"))
 				) {
 				strcpy(relevant_partnum, mime_partnum);
 			}
@@ -882,21 +1049,28 @@ void display_using_handler(long msgnum, int unread,
 			cal = icalcomponent_new_from_string(relevant_source);
 			if (cal != NULL) {
 
-				ical_dezonify(cal);
-
-				/** Simple components of desired type */
-				if (icalcomponent_isa(cal) == which_kind) {
-					callback(cal, msgnum, from, unread);
+				/* A which_kind of (-1) means just load the whole thing */
+				if (which_kind == (-1)) {
+					callback(cal, msgnum, from, unread, calv);
 				}
 
-				/** Subcomponents of desired type */
-				for (c = icalcomponent_get_first_component(cal,
-									   which_kind);
-				     (c != 0);
-				     c = icalcomponent_get_next_component(cal,
-									  which_kind)) {
-					callback(c, msgnum, from, unread);
+				/* Otherwise recurse and hunt */
+				else {
+
+					/* Simple components of desired type */
+					if (icalcomponent_isa(cal) == which_kind) {
+						callback(cal, msgnum, from, unread, calv);
+					}
+	
+					/* Subcomponents of desired type */
+					for (c = icalcomponent_get_first_component(cal, which_kind);
+				     	(c != 0);
+				     	c = icalcomponent_get_next_component(cal, which_kind)) {
+						callback(c, msgnum, from, unread, calv);
+					}
+
 				}
+
 				icalcomponent_free(cal);
 			}
 			free(relevant_source);
@@ -905,69 +1079,63 @@ void display_using_handler(long msgnum, int unread,
 	icalmemory_free_ring();
 }
 
-/**
- * \brief display whole calendar
- * \param msgnum number of the mesage in our db
+/*
+ * Display a calendar item
  */
-void display_calendar(long msgnum, int unread) {
-	display_using_handler(msgnum, unread,
-			      ICAL_VEVENT_COMPONENT,
-			      display_individual_cal);
+void load_calendar_item(message_summary *Msg, int unread, struct calview *c) {
+	/*load_ical_object(Msg->msgnum, unread, ICAL_VEVENT_COMPONENT, display_individual_cal, c);*/
+	load_ical_object(Msg->msgnum, unread, (-1), display_individual_cal, c);
 }
 
-/**
- * \brief display whole taksview
- * \param msgnum number of the mesage in our db
+/*
+ * Display task view
  */
-void display_task(long msgnum, int unread) {
-	display_using_handler(msgnum, unread,
-			      ICAL_VTODO_COMPONENT,
-			      display_individual_cal);
+void display_task(message_summary *Msg, int unread) {
+	load_ical_object(Msg->msgnum, unread, ICAL_VTODO_COMPONENT, display_individual_cal, NULL);
 }
 
-/**
- * \brief display the editor component for a task
+/*
+ * Display the editor component for a task
  */
 void display_edit_task(void) {
 	long msgnum = 0L;
 			
-	/** Force change the room if we have to */
+	/* Force change the room if we have to */
 	if (havebstr("taskrm")) {
-		gotoroom((char *)bstr("taskrm"));
+		gotoroom(sbstr("taskrm"));
 	}
 
 	msgnum = lbstr("msgnum");
 	if (msgnum > 0L) {
-		/** existing task */
-		display_using_handler(msgnum, 0,
+		/* existing task */
+		load_ical_object(msgnum, 0,
 				      ICAL_VTODO_COMPONENT,
-				      display_edit_individual_task);
+				      display_edit_individual_task,
+				      NULL
+		);
 	}
 	else {
-		/** new task */
-		display_edit_individual_task(NULL, 0L, "", 0);
+		/* new task */
+		display_edit_individual_task(NULL, 0L, "", 0, NULL);
 	}
 }
 
-/**
- *\brief save an edited task
+/*
+ * save an edited task
  */
 void save_task(void) {
 	long msgnum = 0L;
-
 	msgnum = lbstr("msgnum");
 	if (msgnum > 0L) {
-		display_using_handler(msgnum, 0,
-				      ICAL_VTODO_COMPONENT,
-				      save_individual_task);
+		load_ical_object(msgnum, 0, ICAL_VTODO_COMPONENT, save_individual_task, NULL);
 	}
 	else {
-		save_individual_task(NULL, 0L, "", 0);
+		save_individual_task(NULL, 0L, "", 0, NULL);
 	}
 }
 
-/**
- * \brief display the editor component for an event
+/*
+ * display the editor component for an event
  */
 void display_edit_event(void) {
 	long msgnum = 0L;
@@ -975,18 +1143,16 @@ void display_edit_event(void) {
 	msgnum = lbstr("msgnum");
 	if (msgnum > 0L) {
 		/* existing event */
-		display_using_handler(msgnum, 0,
-				      ICAL_VEVENT_COMPONENT,
-				      display_edit_individual_event);
+		load_ical_object(msgnum, 0, ICAL_VEVENT_COMPONENT, display_edit_individual_event, NULL);
 	}
 	else {
 		/* new event */
-		display_edit_individual_event(NULL, 0L, "", 0);
+		display_edit_individual_event(NULL, 0L, "", 0, NULL);
 	}
 }
 
-/**
- * \brief save an edited event
+/*
+ * save an edited event
  */
 void save_event(void) {
 	long msgnum = 0L;
@@ -994,12 +1160,11 @@ void save_event(void) {
 	msgnum = lbstr("msgnum");
 
 	if (msgnum > 0L) {
-		display_using_handler(msgnum, 0,
-				      ICAL_VEVENT_COMPONENT,
-				      save_individual_event);
+		/* load_ical_object(msgnum, 0, ICAL_VEVENT_COMPONENT, save_individual_event, NULL); */
+		load_ical_object(msgnum, 0, (-1), save_individual_event, NULL);
 	}
 	else {
-		save_individual_event(NULL, 0L, "", 0);
+		save_individual_event(NULL, 0L, "", 0, NULL);
 	}
 }
 
@@ -1007,15 +1172,14 @@ void save_event(void) {
 
 
 
-/**
- * \brief freebusy display (for client software)
- * \param req dunno. ?????
+/*
+ * Anonymous request of freebusy data for a user
  */
-void do_freebusy(char *req) {
+void do_freebusy(const char *req) {
 	char who[SIZ];
 	char buf[SIZ];
-	char *fb;
 	int len;
+	long lines;
 
 	extract_token(who, req, 1, ' ', sizeof who);
 	if (!strncasecmp(who, "/freebusy/", 10)) {
@@ -1035,16 +1199,34 @@ void do_freebusy(char *req) {
 	serv_getln(buf, sizeof buf);
 
 	if (buf[0] != '1') {
-		wprintf("HTTP/1.1 404 %s\n", &buf[4]);
+		hprintf("HTTP/1.1 404 %s\n", &buf[4]);
 		output_headers(0, 0, 0, 0, 0, 0);
-		wprintf("Content-Type: text/plain\r\n");
-		wprintf("\r\n");
+		hprintf("Content-Type: text/plain\r\n");
 		wprintf("%s\n", &buf[4]);
+		end_burst();
 		return;
 	}
 
-	fb = read_server_text();
-	http_transmit_thing(fb, strlen(fb), "text/calendar", 0);
-	free(fb);
+	read_server_text(WC->WBuf, &lines);
+	http_transmit_thing("text/calendar", 0);
 }
 
+
+
+
+
+void 
+InitModule_CALENDAR
+(void)
+{
+	RegisterPreference("daystart", _("Calendar day view begins at:"), PRF_INT, NULL);
+	RegisterPreference("dayend", _("Calendar day view ends at:"), PRF_INT, NULL);
+	RegisterPreference("weekstart", _("Week starts on:"), PRF_INT, NULL);
+
+	WebcitAddUrlHandler(HKEY("display_edit_task"), display_edit_task, 0);
+	WebcitAddUrlHandler(HKEY("save_task"), save_task, 0);
+	WebcitAddUrlHandler(HKEY("display_edit_event"), display_edit_event, 0);
+	WebcitAddUrlHandler(HKEY("save_event"), save_event, 0);
+	WebcitAddUrlHandler(HKEY("respond_to_request"), respond_to_request, 0);
+	WebcitAddUrlHandler(HKEY("handle_rsvp"), handle_rsvp, 0);
+}
