@@ -1,21 +1,13 @@
 /*
- * $Id
+ * $Id: gettext.c 7169 2009-02-26 22:55:28Z dothebart $
  */
-/**
- * \defgroup LocaleHeaderParser Parse the browser http locale headers and set the NLS stuff.
- * \ingroup WebcitHttpServer 
- */
-/*@{*/
+
 #include "webcit.h"
 #include "webserver.h"
 
 #ifdef ENABLE_NLS
-
-#define NUM_LANGS 10 /**< how many different locales do we know? */
-#define SEARCH_LANG 20 /**< how many langs should we parse? */
-
-/** actual supported locales */
-char *AvailLang[NUM_LANGS] = {
+/* actual supported locales */
+const char *AvailLang[NUM_LANGS] = {
 	"C",
 	"en_US",
 	"de_DE",
@@ -28,6 +20,9 @@ char *AvailLang[NUM_LANGS] = {
 	"pt_BR"
 };
 
+const char *AvailLangLoaded[NUM_LANGS];
+long nLocalesLoaded = 0;
+
 #ifdef HAVE_USELOCALE
 locale_t wc_locales[NUM_LANGS]; /**< here we keep the parsed stuff */
 #endif
@@ -52,7 +47,7 @@ typedef struct _lang_pref{
  * \param LocaleString the string from the browser http headers
  */
 
-void httplang_to_locale(char *LocaleString)
+void httplang_to_locale(StrBuf *LocaleString)
 {
 	LangStruct wanted_locales[SEARCH_LANG];
 	LangStruct *ls;
@@ -64,70 +59,79 @@ void httplang_to_locale(char *LocaleString)
 	int av;
 	int nBest;
 	int nParts;
-	char search[1024];
+	StrBuf *Buf = NULL;
+	StrBuf *SBuf = NULL;
 	
-	safestrncpy(search, LocaleString, sizeof search);
-	nParts=num_tokens(search,',');
+	nParts=StrBufNum_tokens(LocaleString,',');
 	for (i=0; ((i<nParts)&&(i<SEARCH_LANG)); i++)
         {
-			char buf[16];
-			char sbuf[16];
-			char lbuf[16];
-			int blen;
+		char lbuf[16];
+		int blen;
 			
-			ls=&wanted_locales[i];
+		if (Buf == NULL) {
+			Buf = NewStrBuf();
+			SBuf = NewStrBuf();
+		}
+		else {
+			FlushStrBuf(Buf);
+			FlushStrBuf(SBuf);
+		}
 
-			extract_token(&buf[0],search, i,',',16);
-			/** we are searching, if this list item has something like ;q=n*/
-			if (num_tokens(&buf[0],'=')>1) {
-				int sbuflen, k;
-				extract_token(&sbuf[0],&buf[0], 1,'=',16);
-				sbuflen=strlen(&sbuf[0]);
-				for (k=0; k<sbuflen; k++) if (sbuf[k]=='.') sbuf[k]='0';
-				ls->priority=atol(&sbuf[0]);
-			}
-			else {
-				ls->priority=1000;
-			}
-			/** get the locale part */
-			extract_token(&sbuf[0],&buf[0],0,';',16);
-			/** get the lang part, which should be allways there */
-			extract_token(&ls->lang[0],&sbuf[0],0,'-',16);
-			/** get the area code if any. */
-			if (num_tokens(&sbuf[0],'-')>1) {
-				extract_token(&ls->region[0],&sbuf[0],1,'-',16);
-			}
-			else { /** no ara code? use lang code */
-				blen=strlen(&ls->lang[0]);
-				memcpy(&ls->region[0], ls->lang,blen);
-				ls->region[blen]='\0';
-			} /** area codes are uppercase */
-			blen=strlen(&ls->region[0]);
-			for (j=0; j<blen; j++)
-				{
-					int chars=toupper(ls->region[j]);
-					ls->region[j]=(char)chars;/** \todo ?! */
-				}
-			sprintf(&lbuf[0],"%s_%s",&ls->lang[0],&ls->region[0]);
+		ls=&wanted_locales[i];
+
+		StrBufExtract_token(Buf,LocaleString, i,',');
+		/** we are searching, if this list item has something like ;q=n*/
+		if (StrBufNum_tokens(Buf,'=')>1) {
+			int sbuflen, k;
+			StrBufExtract_token(SBuf,Buf, 1,'=');
+			sbuflen=StrLength(SBuf);
+			for (k=0; k<sbuflen; k++) 
+				if (ChrPtr(SBuf)[k]=='.') 
+					StrBufPeek(SBuf, NULL, k, '0');
+			ls->priority=StrTol(SBuf);
+		}
+		else {
+			ls->priority=1000;
+		}
+		/** get the locale part */
+		StrBufExtract_token(SBuf ,Buf, 0, ';');
+		/** get the lang part, which should be allways there */
+		extract_token(&ls->lang[0], ChrPtr(SBuf), 0, '-', 16);
+		/** get the area code if any. */
+		if (StrBufNum_tokens(SBuf,'-') > 1) {
+			extract_token(&ls->region[0],ChrPtr(SBuf),1,'-',16);
+		}
+		else { /** no ara code? use lang code */
+			blen=strlen(&ls->lang[0]);
+			memcpy(&ls->region[0], ls->lang,blen);
+			ls->region[blen]='\0';
+		} /** area codes are uppercase */
+		blen=strlen(&ls->region[0]);
+		for (j=0; j<blen; j++)
+		{
+			int chars=toupper(ls->region[j]);
+			ls->region[j]=(char)chars;/** \todo ?! */
+		}
+		sprintf(&lbuf[0],"%s_%s",&ls->lang[0],&ls->region[0]);
 			
-			/** check if we have this lang */
-			ls->availability=1;
-			ls->selectedlang=-1;
-			for (j=0; j<NUM_LANGS; j++) {
-				int result;
-				/** match against the LANG part */
-				result=strcasecmp(&ls->lang[0], AvailLang[j]);
-				if ((result<0)&&(result<ls->availability)){
-					ls->availability=result;
-					ls->selectedlang=j;
-				}
-				/** match against lang and locale */
-				if (0==strcasecmp(&lbuf[0], AvailLang[j])){
-					ls->availability=0;
-					ls->selectedlang=j;
-					j=NUM_LANGS;
-				}
+		/** check if we have this lang */
+		ls->availability=1;
+		ls->selectedlang=-1;
+		for (j=0; j<nLocalesLoaded; j++) {
+			int result;
+			/** match against the LANG part */
+			result=strcasecmp(&ls->lang[0], AvailLangLoaded[j]);
+			if ((result<0)&&(result<ls->availability)){
+				ls->availability=result;
+				ls->selectedlang=j;
 			}
+			/** match against lang and locale */
+			if (0==strcasecmp(&lbuf[0], AvailLangLoaded[j])){
+				ls->availability=0;
+				ls->selectedlang=j;
+				j=nLocalesLoaded;
+			}
+		}
         }
 	
 	prio=0;
@@ -149,53 +153,18 @@ void httplang_to_locale(char *LocaleString)
 		nBest=0;
 	}
 	WC->selected_language=nBest;
-	lprintf(9, "language found: %s\n", AvailLang[WC->selected_language]);
+	lprintf(9, "language found: %s\n", AvailLangLoaded[WC->selected_language]);
+	FreeStrBuf(&Buf);
+	FreeStrBuf(&SBuf);
 }
 
-/* TODO: we skip the language weighting so far. */
-/* Accept-Language: 'de-de,en-us;q=0.7,en;q=0.3' */
-/* Accept-Language: de,en-ph;q=0.8,en-us;q=0.5,de-at;q=0.3 */
-//void httplang_to_locale(char *LocaleString)
-//{
-//	char selected_locale[16];
-//	int i, j;
-//	char lang[64];
-//	int num_accept = 0;
-//
-//	lprintf(9, "languageAccept: %s\n", LocaleString);
-//
-//	strcpy(selected_locale, "C");
-//	num_accept = num_tokens(LocaleString, ',');
-//
-//	for (i=num_accept-1; i>=0; --i) {
-//		extract_token(lang, LocaleString, i, ',', sizeof lang);
-//
-//		/* Strip out the weights; we don't use them.  Also convert
-//		 * hyphens to underscores.
-//		 */
-//		for (j=0; j<strlen(lang); ++j) {
-//			if (lang[j] == '-') lang[j] = '_';
-//			if (lang[j] == ';') lang[j] = 0;
-//		}
-//
-//		for (j=0; j<NUM_LANGS; ++j) {
-//			if (!strncasecmp(lang, AvailLang[j], strlen(lang))) {
-//				strcpy(selected_locale, AvailLang[j]);
-//			}
-//		}
-//	}
-//
-//	lprintf(9, "language found: %s\n", selected_locale);
-//	set_selected_language(selected_locale);
-//}
-
-
 /**
  * \brief show the language chooser on the login dialog
  * depending on the browser locale change the sequence of the 
  * language chooser.
  */
-void offer_languages(void) {
+void tmplput_offer_languages(StrBuf *Target, WCTemplputParams *TP)
+{
 	int i;
 #ifndef HAVE_USELOCALE
 	char *Lang = getenv("LANG");
@@ -204,16 +173,22 @@ void offer_languages(void) {
 		Lang = "C";
 #endif
 
+
+	if (nLocalesLoaded == 1) {
+		wprintf("<p>%s</p>", AvailLangLoaded[0]);
+		return;
+	}
+
 	wprintf("<select name=\"language\" id=\"lname\" size=\"1\">\n");
 
-	for (i=0; i < NUM_LANGS; ++i) {
+	for (i=0; i < nLocalesLoaded; ++i) {
 #ifndef HAVE_USELOCALE
-		if (strcmp(AvailLang[i], Lang) == 0)
+		if (strcmp(AvailLangLoaded[i], Lang) == 0)
 #endif
 		wprintf("<option %s value=%s>%s</option>\n",
 			((WC->selected_language == i) ? "selected" : ""),
-			AvailLang[i],
-			AvailLang[i]
+			AvailLangLoaded[i],
+			AvailLangLoaded[i]
 		);
 	}
 
@@ -224,11 +199,11 @@ void offer_languages(void) {
  * \brief Set the selected language for this session.
  * \param lang the locale to set.
  */
-void set_selected_language(char *lang) {
-	int i;
+void set_selected_language(const char *lang) {
 #ifdef HAVE_USELOCALE
-	for (i=0; i<NUM_LANGS; ++i) {
-		if (!strcasecmp(lang, AvailLang[i])) {
+	int i;
+	for (i=0; i<nLocalesLoaded; ++i) {
+		if (!strcasecmp(lang, AvailLangLoaded[i])) {
 			WC->selected_language = i;
 		}
 	}
@@ -240,8 +215,9 @@ void set_selected_language(char *lang) {
  */
 void go_selected_language(void) {
 #ifdef HAVE_USELOCALE
-	if (WC->selected_language < 0) return;
-	uselocale(wc_locales[WC->selected_language]);	/** switch locales */
+	wcsession *WCC = WC;
+	if (WCC->selected_language < 0) return;
+	uselocale(wc_locales[WCC->selected_language]);	/** switch locales */
 	textdomain(textdomain(NULL));			/** clear the cache */
 #else
 	char *language;
@@ -261,46 +237,48 @@ void stop_selected_language(void) {
 #endif
 }
 
-void preset_locale(void)
-{
-#ifndef HAVE_USELOCALE
-#ifdef HAVE_GETTEXT
-	char *language;
-	
-	lprintf(9, "Nailing locale to %s\n", getenv("LANG"));
-	language = getenv("LANG");
-	setlocale(LC_MESSAGES, language);
+#ifdef HAVE_USELOCALE
+	locale_t Empty_Locale;
 #endif
-#endif
-}
+
 /**
  * \brief Create a locale_t for each available language
  */
 void initialize_locales(void) {
 	int i;
 	char buf[32];
+	char *language = NULL;
+	
+	language = getenv("WEBCIT_LANG");
+	if ((language) && (!IsEmptyStr(language)) && (strcmp(language, "UNLIMITED") != 0)) {
+		lprintf(9, "Nailing locale to %s\n", language);
+ 	}
+	else language = NULL;
 
 #ifdef HAVE_USELOCALE
-	locale_t Empty_Locale;
-
 	/* create default locale */
 	Empty_Locale = newlocale(LC_ALL_MASK, NULL, NULL);
 #endif
 
+
+
+
 	for (i = 0; i < NUM_LANGS; ++i) {
+		if ((language != NULL) && (strcmp(AvailLang[i], language) != 0))
+			continue;
 		if (i == 0) {
-			sprintf(buf, "%s", AvailLang[i]);	// locale 0 (C) is ascii, not utf-8
+			sprintf(buf, "%s", AvailLang[i]);	/* locale 0 (C) is ascii, not utf-8 */
 		}
 		else {
 			sprintf(buf, "%s.UTF8", AvailLang[i]);
 		}
 #ifdef HAVE_USELOCALE
-		wc_locales[i] = newlocale(
+		wc_locales[nLocalesLoaded] = newlocale(
 			(LC_MESSAGES_MASK|LC_TIME_MASK),
 			buf,
 			(((i > 0) && (wc_locales[0] != NULL)) ? wc_locales[0] : Empty_Locale)
 		);
-		if (wc_locales[i] == NULL) {
+		if (wc_locales[nLocalesLoaded] == NULL) {
 			lprintf(1, "Error configuring locale for %s: %s\n",
 				buf,
 				strerror(errno)
@@ -308,15 +286,62 @@ void initialize_locales(void) {
 		}
 		else {
 			lprintf(3, "Configured available locale: %s\n", buf);
+			AvailLangLoaded[nLocalesLoaded] = AvailLang[i];
+			nLocalesLoaded++;
+		}
+#else
+		if (language != NULL) {
+			setenv("LANG", buf, 1);
+			AvailLangLoaded[nLocalesLoaded] = AvailLang[i];
+			nLocalesLoaded++;
+		}
+		else if (nLocalesLoaded == 0) {
+			setenv("LANG", buf, 1);
+			AvailLangLoaded[nLocalesLoaded] = AvailLang[i];
+			nLocalesLoaded++;
 		}
 #endif
 	}
+	if ((language != NULL) && (nLocalesLoaded == 0)) {
+		lprintf(1, "Your selected locale [%s] isn't available on your system. falling back to C\n", language);
+#ifdef HAVE_USELOCALE
+		wc_locales[0] = newlocale(
+			(LC_MESSAGES_MASK|LC_TIME_MASK),
+			AvailLang[0],
+			Empty_Locale);		
+#else
+		setlocale(LC_MESSAGES, AvailLang[0]);
+		setenv("LANG", AvailLang[0], 1);
+#endif
+		AvailLangLoaded[0] = AvailLang[0];
+		nLocalesLoaded = 1;
+	}
+#ifndef HAVE_USELOCALE
+
+
+#endif
+
 }
 
 
+void ShutdownLocale(void)
+{
+#ifdef HAVE_USELOCALE
+	int i;
+	for (i = 0; i < nLocalesLoaded; ++i) {
+		if (Empty_Locale != wc_locales[i])
+			freelocale(wc_locales[i]);
+	}
+#endif
+}
+
 #else	/* ENABLE_NLS */
+const char *AvailLang[NUM_LANGS] = {
+	"C"};
+
 /** \brief dummy for non NLS enabled systems */
-void offer_languages(void) {
+void tmplput_offer_languages(StrBuf *Target, WCTemplputParams *TP)
+{
 	wprintf("English (US)");
 }
 
@@ -332,10 +357,34 @@ void go_selected_language(void) {
 void stop_selected_language(void) {
 }
 
-void preset_locale(void)
-{
-}
 #endif	/* ENABLE_NLS */
 
 
-/*@}*/
+void TmplGettext(StrBuf *Target, WCTemplputParams *TP)
+{
+	StrBufAppendBufPlain(Target, _(TP->Tokens->Params[0]->Start), -1, 0);
+}
+
+
+/*
+ * Returns the language currently in use.
+ * This function returns a static string, so don't do anything stupid please.
+ */
+const char *get_selected_language(void) {
+#ifdef ENABLE_NLS
+#ifdef HAVE_USELOCALE
+	return AvailLang[WC->selected_language];
+#else
+	return "en";
+#endif
+#else
+	return "en";
+#endif
+}
+
+void 
+InitModule_GETTEXT
+(void)
+{
+	RegisterNamespace("LANG:SELECT", 0, 0, tmplput_offer_languages, CTX_NONE);
+}
