@@ -1,5 +1,5 @@
 /*
- * $Id: mainmenu.c 6352 2008-05-31 04:30:22Z ajc $
+ * $Id: mainmenu.c 6993 2009-01-25 21:31:24Z dothebart $
  */
 
 #include "webcit.h"
@@ -9,17 +9,23 @@
  */
 void display_main_menu(void)
 {
+	begin_burst();
+	output_headers(1, 0, 0, 0, 1, 0);
+	DoTemplate(HKEY("display_main_menu"), NULL, &NoCtx);
+	end_burst();
+
+/*
 	char buf[SIZ];
 	output_headers(1, 1, 1, 0, 0, 0);
 
 	wprintf("<div class=\"fix_scrollbar_bug\">"
-		"<table width=\"100%%\" cellspacing=\"10px\" cellpadding=\"0\">"
+		"<table width=\"100%%\" cellspacing=\"10\" cellpadding=\"0\">"
 		"<tr><td colspan=\"2\" class=\"advanced\">\n");
 
 	svput("BOXTITLE", WCS_STRING, _("Basic commands"));
-	do_template("beginbox");
+	do_template("beginbox", NULL);
 
-	/* start of first column */
+	/ * start of first column * /
 	wprintf("<ul class=\"adminitems col1\">");
 
 	wprintf("<li><a href=\"knrooms\">");
@@ -50,7 +56,7 @@ void display_main_menu(void)
 
 	wprintf("</ul>\n");
 
-	/* start of second column */
+	/ * start of second column * /
 
 	wprintf("<ul class=\"adminitems col2\">");
 
@@ -82,7 +88,7 @@ void display_main_menu(void)
 
 	wprintf("</ul>\n");
 
-	/* start of third column */
+	/ * start of third column * /
 
 	wprintf("<ul class=\"adminitems lastcol\">");
 
@@ -108,7 +114,7 @@ void display_main_menu(void)
 
 	wprintf("&nbsp;");
 
-	do_template("endbox");
+	do_template("endbox", NULL);
 
 	wprintf("</td></tr>"
 		"<tr valign=top><td width=50%%>");
@@ -144,6 +150,7 @@ void display_main_menu(void)
 
 	wprintf("</td></tr></table></div>");
 	wDumpContent(2);
+*/
 }
 
 
@@ -152,6 +159,11 @@ void display_main_menu(void)
  */
 void display_aide_menu(void)
 {
+	begin_burst();
+	output_headers(1, 0, 0, 0, 1, 0);
+	DoTemplate(HKEY("display_aide_menu"), NULL, &NoCtx);
+	end_burst();
+/*
 	output_headers(1, 1, 2, 0, 0, 0);
 
         wprintf("<div id=\"banner\">\n");
@@ -193,6 +205,7 @@ void display_aide_menu(void)
 
 	wprintf("</td></tr></table></div>");
 	wDumpContent(2);
+*/
 }
 
 
@@ -221,7 +234,7 @@ void display_generic(void)
 	wprintf("<br />\n");
 
 	wprintf("<form method=\"post\" action=\"do_generic\">\n");
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%ld\">\n", WC->nonce);
+	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
 
 	wprintf(_("Enter command:"));
 	wprintf("<br /><input type=\"text\" name=\"g_cmd\" size=80 maxlength=\"250\"><br />\n");
@@ -230,7 +243,7 @@ void display_generic(void)
 	wprintf("<br /><textarea name=\"g_input\" rows=10 cols=80 width=80></textarea><br />\n");
 
 	wprintf("<font size=-2>");
-	wprintf(_("Detected host header is %s://%s"), (is_https ? "https" : "http"), WC->http_host);
+	wprintf(_("Detected host header is %s://%s"), (is_https ? "https" : "http"), ChrPtr(WC->http_host));
 	wprintf("</font>\n");
 	wprintf("<input type=\"submit\" name=\"sc_button\" value=\"%s\">", _("Send command"));
 	wprintf("&nbsp;");
@@ -262,7 +275,7 @@ void do_generic(void)
 	serv_getln(buf, sizeof buf);
 
 	svput("BOXTITLE", WCS_STRING, _("Server command results"));
-	do_template("beginbox");
+	do_template("beginboxx", NULL);
 
 	wprintf("<table border=0><tr><td>Command:</td><td><tt>");
 	escputs(bstr("g_cmd"));
@@ -300,7 +313,7 @@ void do_generic(void)
 	wprintf("<hr />");
 	wprintf("<a href=\"display_generic\">Enter another command</a><br />\n");
 	wprintf("<a href=\"display_advanced\">Return to menu</a>\n");
-	do_template("endbox");
+	do_template("endbox", NULL);
 	wDumpContent(1);
 }
 
@@ -322,10 +335,10 @@ void display_menubar(int as_single_page) {
 			"body	{ text-decoration: none; }\n"
 			"</style>\n"
 			"</head>\n");
-		do_template("background");
+		do_template("background", NULL);
 	}
 
-	do_template("menubar");
+	do_template("menubar", NULL);
 
 	if (as_single_page) {
 		wDumpContent(2);
@@ -344,74 +357,73 @@ void display_shutdown(void)
 	char *when;
 	
 	when=bstr("when");
-	if (!strcmp(when, "now")){
+	if (strcmp(when, "now") == 0){
 		serv_printf("DOWN 1");
 		serv_getln(buf, sizeof buf);
 		if (atol(buf) == 500)
 		{ /* upsie. maybe the server is not running as daemon? */
-			wprintf("<html><head></head><body>Attention: %s</body></html>", &buf[4]);
-
+			
+			safestrncpy(WC->ImportantMessage,
+				    &buf[4],
+				    sizeof WC->ImportantMessage);
 		}
-		else {
-			wprintf("<html>\n"
-				"<head>\n"
-				"<meta http-equiv=\"refresh\" content=\"15; URL=knrooms\"/>\n"
-				"</head>\n"
-				"<body bgcolor=\"#FFFFFF\">\n"
-				"<img src=\"static/throbber.gif\" /> <font color=\"#AAAAAA\">%s </font>"
-				"</body>\n</html>\n",
-				_("Please wait while the Citadel server is restarted... ")
-				);
-		}
-		wDumpContent(0);
+		begin_burst();
+		output_headers(1, 0, 0, 0, 1, 0);
+		DoTemplate(HKEY("display_serverrestart"), NULL, &NoCtx);
+		end_burst();
 		lingering_close(WC->http_sock);
 		sleeeeeeeeeep(10);
 		serv_printf("NOOP");
 		serv_printf("NOOP");
 	}
-	else if (!strcmp(when, "page")) {
+	else if (strcmp(when, "page") == 0) {
 		char *message;
 	       
 		message = bstr("message");
 		if ((message == NULL) || (IsEmptyStr(message)))
 		{
-			output_headers(1, 1, 1, 0, 0, 0);
-			svput("BOXTITLE", WCS_STRING, _("Message to your Users:"));
-			do_template("beginbox");
-			wprintf("<form action=\"server_shutdown\">\n"
-				"<input type=\"hidden\" name=\"when\" value=\"page\">\n"
-				"<input type=\"text\" name=\"message\" value=\"%s\">\n"
-				"<input type=\"submit\" value=\"go\">\n"
-				"</form>\n",
-				_("The citadel server has to be restarted. It 'll be back in a minute.")
-				);
-			do_template("endbox");
-			wDumpContent(1);
-
-			
+			begin_burst();
+			output_headers(1, 0, 0, 0, 1, 0);
+			DoTemplate(HKEY("display_serverrestartpage"), NULL, &NoCtx);
+			end_burst();
 		}
 		else
 		{
-			// TODO: page the users... wait longer...
 			serv_printf("SEXP broadcast|%s", message);
-			serv_getln(buf, sizeof buf); // should we care?
-			wprintf("<html>\n"
-				"<head>\n"
-				"<meta http-equiv=\"refresh\" content=\"15; URL=knrooms\"/>\n"
-				"</head>\n"
-				"<body bgcolor=\"#FFFFFF\">\n"
-				"<img src=\"static/throbber.gif\" /> <font color=\"#AAAAAA\">%s </font>"
-				"</body>\n</html>\n",
-				_("Please wait while your users are being paged, the citadel server will be restarted after that... "));
-			
+			serv_getln(buf, sizeof buf); /* TODO: should we care? */
+			begin_burst();
+			output_headers(1, 0, 0, 0, 1, 0);
+			DoTemplate(HKEY("display_serverrestartpagedo"), NULL, &NoCtx);
+			end_burst();			
 		}
 	}
 	else if (!strcmp(when, "idle")) {
 		serv_printf("SCDN 3");
 		serv_getln(buf, sizeof buf);
-		if (atol(buf) == 500) {
-			/* oops ... maybe the server is not running as a daemon? */
-			wprintf("<html><head></head><body>Attention: %s</body></html>", &buf[4]);
+
+		if (atol(buf) == 500)
+		{ /* upsie. maybe the server is not running as daemon? */
+			safestrncpy(WC->ImportantMessage,
+				    &buf[4],
+				    sizeof WC->ImportantMessage);
 		}
+		begin_burst();
+		output_headers(1, 0, 0, 0, 1, 0);
+		DoTemplate(HKEY("display_aide_menu"), NULL, &NoCtx);
+		end_burst();			
 	}
 }
+
+void _display_menubar(void) { display_menubar(0); }
+
+void 
+InitModule_MAINMENU
+(void)
+{
+	WebcitAddUrlHandler(HKEY("display_aide_menu"), display_aide_menu, 0);
+	WebcitAddUrlHandler(HKEY("server_shutdown"), display_shutdown, 0);
+	WebcitAddUrlHandler(HKEY("display_main_menu"), display_main_menu, 0);
+	WebcitAddUrlHandler(HKEY("display_generic"), display_generic, 0);
+	WebcitAddUrlHandler(HKEY("do_generic"), do_generic, 0);
+	WebcitAddUrlHandler(HKEY("display_menubar"), _display_menubar, 0);
+}
