@@ -1,5 +1,5 @@
 /*
- * $Id: notes.c 6273 2008-05-12 17:21:34Z ajc $
+ * $Id: notes.c 6952 2009-01-14 23:46:21Z dothebart $
  *
  */
 
@@ -27,7 +27,7 @@ void display_vnote_div(struct vnote *v) {
 	int i;
 
 
-	wprintf("<div id=\"note-%s\" ", v->uid);	// begin outer div
+	wprintf("<div id=\"note-%s\" ", v->uid);	/* begin outer div */
 	wprintf("class=\"stickynote_outer\" ");
 	wprintf("style=\"");
 	wprintf("left: %dpx; ", v->pos_left);
@@ -41,7 +41,7 @@ void display_vnote_div(struct vnote *v) {
 
 
 
-	wprintf("<div id=\"titlebar-%s\" ", v->uid);	// begin title bar div
+	wprintf("<div id=\"titlebar-%s\" ", v->uid);	/* begin title bar div */
 	wprintf("class=\"stickynote_titlebar\" ");
 	wprintf("onMouseDown=\"NotesDragMouseDown(event,'%s')\" ", v->uid);
 	wprintf("style=\"");
@@ -75,8 +75,8 @@ void display_vnote_div(struct vnote *v) {
 	escputs(v->body);
 	wprintf("</div>\n");				// end body div
 
-	wprintf("<script type=\"text/javascript\">");
-	wprintf(" new Ajax.InPlaceEditor('notebody-%s', 'ajax_update_note?note_uid=%s', "
+        StrBufAppendPrintf(WC->trailing_javascript,
+		" new Ajax.InPlaceEditor('notebody-%s', 'ajax_update_note?note_uid=%s', "
 		"{rows:%d,cols:%d,onEnterHover:false,onLeaveHover:false,"
 		"okText:'%s',cancelText:'%s',clickToEditText:'%s'});",
 		v->uid,
@@ -87,9 +87,6 @@ void display_vnote_div(struct vnote *v) {
 		_("Cancel"),
 		_("Click on any note to edit it.")
 	);
-	wprintf("</script>\n");
-
-
 
 	wprintf("<div id=\"resize-%s\" ", v->uid);	// begin resize handle div
 	wprintf("class=\"stickynote_resize\" ");
@@ -275,7 +272,7 @@ void ajax_update_note(void) {
 	// Was this request a delete operation?  If so, nuke it...
 	if (havebstr("deletenote")) {
 		if (!strcasecmp(bstr("deletenote"), "yes")) {
-			serv_printf("DELE %ld", msgnum);
+			serv_printf("DELE %d", msgnum);
 			serv_getln(buf, sizeof buf);
 			begin_ajax_response();
 			wprintf("%s", buf);
@@ -288,7 +285,7 @@ void ajax_update_note(void) {
 	v = vnote_new_from_msg(msgnum);
 	if (!v) {
 		begin_ajax_response();
-		wprintf("Cannot locate a vNote within message %ld\n", msgnum);
+		wprintf("Cannot locate a vNote within message %d\n", msgnum);
 		end_ajax_response();
 		return;
 	}
@@ -340,20 +337,29 @@ void ajax_update_note(void) {
  *
  * msgnum = Message number on the local server of the note to be displayed
  */
-void display_note(long msgnum, int unread) {
+void display_note(message_summary *Msg, int unread) {
 	struct vnote *v;
+	WCTemplputParams TP;
 
-	v = vnote_new_from_msg(msgnum);
+	memset(&TP, 0, sizeof(WCTemplputParams));
+	TP.Filter.ContextType = CTX_VNOTE;
+	v = vnote_new_from_msg(Msg->msgnum);
 	if (v) {
-		display_vnote_div(v);
+//		display_vnote_div(v);
+		TP.Context = v;
+		DoTemplate(HKEY("vnoteitem"),
+			   WC->WBuf, &TP);
+			
 
 		/* uncomment these lines to see ugly debugging info 
-		wprintf("<script type=\"text/javascript\">");
-		wprintf("document.write('L: ' + $('note-%s').style.left + '; ');", v->uid);
-		wprintf("document.write('T: ' + $('note-%s').style.top + '; ');", v->uid);
-		wprintf("document.write('W: ' + $('note-%s').style.width + '; ');", v->uid);
-		wprintf("document.write('H: ' + $('note-%s').style.height + '<br>');", v->uid);
-		wprintf("</script>");
+		StrBufAppendPrintf(WC->trailing_javascript,
+			"document.write('L: ' + $('note-%s').style.left + '; ');", v->uid);
+		StrBufAppendPrintf(WC->trailing_javascript,
+			"document.write('T: ' + $('note-%s').style.top + '; ');", v->uid);
+		StrBufAppendPrintf(WC->trailing_javascript,
+			"document.write('W: ' + $('note-%s').style.width + '; ');", v->uid);
+		StrBufAppendPrintf(WC->trailing_javascript,
+			"document.write('H: ' + $('note-%s').style.height + '<br>');", v->uid);
 		*/
 
 		vnote_free(v);
@@ -380,5 +386,85 @@ void add_new_note(void) {
 		vnote_free(v);
 	}
 	
-	readloop("readfwd");
+	readloop(readfwd);
+}
+
+
+void tmpl_vcard_put_posleft(StrBuf *Target, WCTemplputParams *TP)
+{
+	struct vnote *v = (struct vnote *) CTX;
+	StrBufAppendPrintf(Target, "%d", v->pos_left);
+}
+
+void tmpl_vcard_put_postop(StrBuf *Target, WCTemplputParams *TP)
+{
+	struct vnote *v = (struct vnote *) CTX;
+	StrBufAppendPrintf(Target, "%d", v->pos_top);
+}
+
+void tmpl_vcard_put_poswidth(StrBuf *Target, WCTemplputParams *TP)
+{
+	struct vnote *v = (struct vnote *) CTX;
+	StrBufAppendPrintf(Target, "%d", v->pos_width);
+}
+
+void tmpl_vcard_put_posheight(StrBuf *Target, WCTemplputParams *TP)
+{
+	struct vnote *v = (struct vnote *) CTX;
+	StrBufAppendPrintf(Target, "%d", v->pos_height);
+}
+
+void tmpl_vcard_put_posheight2(StrBuf *Target, WCTemplputParams *TP)
+{
+	struct vnote *v = (struct vnote *) CTX;
+	StrBufAppendPrintf(Target, "%d", (v->pos_height / 16) - 5);
+}
+
+void tmpl_vcard_put_width2(StrBuf *Target, WCTemplputParams *TP)
+{
+	struct vnote *v = (struct vnote *) CTX;
+	StrBufAppendPrintf(Target, "%d", (v->pos_width / 9) - 1);
+}
+
+void tmpl_vcard_put_color(StrBuf *Target, WCTemplputParams *TP)
+{
+	struct vnote *v = (struct vnote *) CTX;
+	StrBufAppendPrintf(Target, "%02X%02X%02X", v->color_red, v->color_green, v->color_blue);
+}
+
+void tmpl_vcard_put_bgcolor(StrBuf *Target, WCTemplputParams *TP)
+{
+	struct vnote *v = (struct vnote *) CTX;
+	StrBufAppendPrintf(Target, "%02X%02X%02X", v->color_red/2, v->color_green/2, v->color_blue/2);
+}
+
+void tmpl_vcard_put_message(StrBuf *Target, WCTemplputParams *TP)
+{
+	struct vnote *v = (struct vnote *) CTX;
+	StrEscAppend(Target, NULL, v->body, 0, 0); ///TODO?
+}
+
+void tmpl_vcard_put_uid(StrBuf *Target, WCTemplputParams *TP)
+{
+	struct vnote *v = (struct vnote *) CTX;
+	StrBufAppendBufPlain(Target, v->uid, -1, 0);
+}
+
+void 
+InitModule_NOTES
+(void)
+{
+	WebcitAddUrlHandler(HKEY("add_new_note"), add_new_note, 0);
+	WebcitAddUrlHandler(HKEY("ajax_update_note"), ajax_update_note, 0);
+
+	RegisterNamespace("VNOTE:POS:LEFT", 0, 0, tmpl_vcard_put_posleft, CTX_VNOTE);
+	RegisterNamespace("VNOTE:POS:TOP", 0, 0, tmpl_vcard_put_postop, CTX_VNOTE);
+	RegisterNamespace("VNOTE:POS:WIDTH", 0, 0, tmpl_vcard_put_poswidth, CTX_VNOTE);
+	RegisterNamespace("VNOTE:POS:HEIGHT", 0, 0, tmpl_vcard_put_posheight, CTX_VNOTE);
+	RegisterNamespace("VNOTE:POS:HEIGHT2", 0, 0, tmpl_vcard_put_posheight2, CTX_VNOTE);
+	RegisterNamespace("VNOTE:POS:WIDTH2", 0, 0, tmpl_vcard_put_width2, CTX_VNOTE);
+	RegisterNamespace("VNOTE:COLOR", 0, 0, tmpl_vcard_put_color, CTX_VNOTE);
+	RegisterNamespace("VNOTE:BGCOLOR", 0, 0,tmpl_vcard_put_bgcolor, CTX_VNOTE);
+	RegisterNamespace("VNOTE:MSG", 0, 1, tmpl_vcard_put_message, CTX_VNOTE);
+	RegisterNamespace("VNOTE:UID", 0, 0, tmpl_vcard_put_uid, CTX_VNOTE);
 }
