@@ -1,15 +1,11 @@
 /*
- * $Id: paging.c 6179 2008-04-04 10:44:32Z dothebart $
+ * $Id: paging.c 7231 2009-03-19 04:32:15Z ajc $
  */
-/**
- * \defgroup PageFunc Functions which implement the chat and paging facilities.
- * \ingroup ClientPower
- */
-/*@{*/
+
 #include "webcit.h"
 
-/**
- * \brief display the form for paging (x-messaging) another user
+/*
+ * display the form for paging (x-messaging) another user
  */
 void display_page(void)
 {
@@ -34,7 +30,8 @@ void display_page(void)
 	wprintf("<br>\n");
 
 	wprintf("<FORM METHOD=\"POST\" action=\"page_user\">\n");
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%ld\">\n", WC->nonce);
+	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
+	wprintf("<input type=\"hidden\" name=\"template\" value=\"who\">\n");
 
 	wprintf("<TABLE border=0 width=100%%><TR><TD>\n");
 
@@ -93,7 +90,7 @@ void page_user(void)
 		}
 	}
 
-	who();
+	url_do_template();
 }
 
 
@@ -106,10 +103,13 @@ void do_chat(void)
 	char buf[SIZ];
 
 	/** First, check to make sure we're still allowed in this room. */
-	serv_printf("GOTO %s", WC->wc_roomname);
+	serv_printf("GOTO %s", ChrPtr(WC->wc_roomname));
 	serv_getln(buf, sizeof buf);
 	if (buf[0] != '2') {
-		smart_goto("_BASEROOM_");
+		StrBuf *Buf;
+		Buf = NewStrBufPlain(HKEY("_BASEROOM_"));
+		smart_goto(Buf);
+		FreeStrBuf(&Buf);
 		return;
 	}
 
@@ -128,7 +128,7 @@ void do_chat(void)
 	 * output by begin_ajax_response() happen to be the ones we need.)
 	 */
 	begin_ajax_response();
-	do_template("chatframeset");
+	do_template("chatframeset", NULL);
 	end_ajax_response();
 	return;
 }
@@ -141,6 +141,7 @@ void do_chat(void)
  */
 void page_popup(void)
 {
+	int len;
 	char buf[SIZ];
 
 	/** JavaScript function to alert the user that popups are probably blocked */
@@ -157,8 +158,8 @@ void page_popup(void)
 
 	/** First, do the check as part of our page load. */
 	serv_puts("NOOP");
-	serv_getln(buf, sizeof buf);
-	if (buf[3] == '*') {
+	len = serv_getln(buf, sizeof buf);
+	if ((len >= 3) && (buf[3] == '*')) {
 		if ((time(NULL) - WC->last_pager_check) > 60) {
 			wprintf("<script type=\"text/javascript\">"
 				" var oWin = window.open('static/instant_messenger.html', "
@@ -227,13 +228,13 @@ int setup_chat_socket(void) {
 
 		serv_getln(buf, sizeof buf);
 		if (buf[0] == '2') {
-			serv_printf("USER %s", WC->wc_username);
+			serv_printf("USER %s", ChrPtr(WC->wc_username));
 			serv_getln(buf, sizeof buf);
 			if (buf[0] == '3') {
-				serv_printf("PASS %s", WC->wc_password);
+				serv_printf("PASS %s", ChrPtr(WC->wc_password));
 				serv_getln(buf, sizeof buf);
 				if (buf[0] == '2') {
-					serv_printf("GOTO %s", WC->wc_roomname);
+					serv_printf("GOTO %s", ChrPtr(WC->wc_roomname));
 					serv_getln(buf, sizeof buf);
 					if (buf[0] == '2') {
 						serv_puts("CHAT");
@@ -277,8 +278,7 @@ void chat_recv(void) {
 
 	output_headers(0, 0, 0, 0, 0, 0);
 
-	wprintf("Content-type: text/html; charset=utf-8\n");
-	wprintf("\n");
+	hprintf("Content-type: text/html; charset=utf-8\r\n");
 	wprintf("<html>\n"
 		"<head>\n"
 		"<meta http-equiv=\"refresh\" content=\"3\" />\n"
@@ -303,7 +303,7 @@ void chat_recv(void) {
 		pf.fd = WC->chat_sock;
 		pf.events = POLLIN;
 		pf.revents = 0;
-		if (poll(&pf, 1, 1) > 0) if (pf.revents & POLLIN) {
+		if ((poll(&pf, 1, 1) > 0) && (pf.revents & POLLIN)) {
 			++got_data;
 
 			/** Temporarily swap the serv and chat sockets during chat talk */
@@ -381,7 +381,7 @@ void chat_recv(void) {
 				if (strcasecmp(cl_user, WC->last_chat_user)) {
 					wprintf("<B>");
 	
-					if (!strcasecmp(cl_user, WC->wc_fullname)) {
+					if (!strcasecmp(cl_user, ChrPtr(WC->wc_fullname))) {
 						wprintf("<FONT COLOR=&quot;#FF0000&quot;>");
 					}
 					else {
@@ -425,8 +425,7 @@ void chat_send(void) {
 	char buf[SIZ];
 
 	output_headers(0, 0, 0, 0, 0, 0);
-	wprintf("Content-type: text/html; charset=utf-8\n");
-	wprintf("\n");
+	hprintf("Content-type: text/html; charset=utf-8\r\n");
 	wprintf("<HTML>"
 		"<BODY onLoad=\"document.chatsendform.send_this.focus();\" >"
 	);
@@ -484,7 +483,7 @@ void chat_send(void) {
 	WC->chat_sock = i;
 
 	wprintf("<FORM METHOD=\"POST\" action=\"chat_send\" NAME=\"chatsendform\">\n");
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%ld\">\n", WC->nonce);
+	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
 	wprintf("<INPUT TYPE=\"text\" SIZE=\"80\" MAXLENGTH=\"%d\" "
 		"NAME=\"send_this\">\n", SIZ-10);
 	wprintf("<br />");
@@ -498,4 +497,33 @@ void chat_send(void) {
 	wDumpContent(0);
 }
 
-/*@}*/
+
+void ajax_send_instant_message(void) {
+	char recp[256];
+	char buf[256];
+
+	safestrncpy(recp, bstr("recp"), sizeof recp);
+
+	serv_printf("SEXP %s|-", recp);
+	serv_getln(buf, sizeof buf);
+
+	if (buf[0] == '4') {
+		text_to_server(bstr("msg"));
+		serv_puts("000");
+	}
+
+	escputs(buf);	/* doesn't really matter what we return - the client ignores it */
+}
+
+
+void 
+InitModule_PAGING
+(void)
+{
+	WebcitAddUrlHandler(HKEY("display_page"), display_page, 0);
+	WebcitAddUrlHandler(HKEY("page_user"), page_user, 0);
+	WebcitAddUrlHandler(HKEY("chat"), do_chat, 0);
+	WebcitAddUrlHandler(HKEY("chat_recv"), chat_recv, 0);
+	WebcitAddUrlHandler(HKEY("chat_send"), chat_send, 0);
+	WebcitAddUrlHandler(HKEY("ajax_send_instant_message"), ajax_send_instant_message, AJAX);
+}
