@@ -1,5 +1,5 @@
 /*
- * $Id: setup_wizard.c 5377 2007-08-10 20:41:22Z thierry $
+ * $Id: setup_wizard.c 7087 2009-02-10 18:50:50Z dothebart $
  *
  * First-time setup wizard
  */
@@ -19,7 +19,7 @@ void do_setup_wizard(void)
 	if (!strcasecmp(step, "Finish")) {
 		fp = fopen(wizard_filename, "w");
 		if (fp != NULL) {
-			fprintf(fp, "%d\n", serv_info.serv_rev_level);
+			fprintf(fp, "%d\n", WC->serv_info->serv_rev_level);
 			fclose(fp);
 		}
 		do_welcome();
@@ -38,7 +38,7 @@ void do_setup_wizard(void)
 	wprintf("<div class=\"fix_scrollbar_bug\">"
 		"<form method=\"post\" action=\"setup_wizard\">\n"
 	);
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%ld\">\n", WC->nonce);
+	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
 
 	wprintf("<div align=center>"
 		"This is where the setup wizard will be placed.<br>\n"
@@ -52,4 +52,9 @@ void do_setup_wizard(void)
 	wDumpContent(1);
 }
 
-
+void 
+InitModule_SETUP_WIZARD
+(void)
+{
+	WebcitAddUrlHandler(HKEY("setup_wizard"), do_setup_wizard, 0);
+}
