@@ -1,5 +1,5 @@
 /* 
- * $Id: sieve.c 6245 2008-05-02 12:10:49Z dothebart $
+ * $Id: sieve.c 7087 2009-02-10 18:50:50Z dothebart $
  */
 
 #include "webcit.h"
@@ -49,7 +49,7 @@ void display_sieve(void)
 	int i;
 	int rules_script_is_active = 0;
 
-	if (!serv_info.serv_supports_sieve) {
+	if (!WC->serv_info->serv_supports_sieve) {
 		display_no_sieve();
 		return;
 	}
@@ -116,7 +116,7 @@ void display_sieve(void)
 
 
 	wprintf("<form id=\"sieveform\" method=\"post\" action=\"save_sieve\">\n");
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%ld\">\n", WC->nonce);
+	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
 
 	wprintf(_("When new mail arrives: "));
         wprintf("<select name=\"bigaction\" size=1 onChange=\"ToggleSievePanels();\">\n");
@@ -355,6 +355,13 @@ void output_sieve_rule(char *hfield, char *compare, char *htext, char *sizecomp,
 		);
 	}
 
+	else if (!strcasecmp(hfield, "listid")) {
+		serv_printf("if%s header %s \"List-ID\" \"%s\"",
+			comp1, comp2,
+			htext
+		);
+	}
+
 	else if (!strcasecmp(hfield, "envfrom")) {
 		serv_printf("if%s envelope %s \"From\" \"%s\"",
 			comp1, comp2,
@@ -539,7 +546,7 @@ void parse_fields_from_rule_editor(void) {
 			serv_printf("# WEBCIT_RULE|%d|%s|", i, encoded_rule);
 			output_sieve_rule(hfield, compare, htext, sizecomp, sizeval,
 					action, fileinto, redirect, automsg, final, my_addresses);
-			serv_printf("");
+			serv_puts("");
 		}
 
 
@@ -649,32 +656,32 @@ void display_add_remove_scripts(char *message)
 	wprintf("<table border=0 cellspacing=10><tr valign=top><td>\n");
 
 	svput("BOXTITLE", WCS_STRING, _("Add a new script"));
-	do_template("beginbox");
+	do_template("beginboxx", NULL);
 
 	wprintf(_("To create a new script, enter the desired "
 		"script name in the box below and click 'Create'."));
 	wprintf("<br /><br />");
 
         wprintf("<center><form method=\"POST\" action=\"create_script\">\n");
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%ld\">\n", WC->nonce);
+	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
         wprintf(_("Script name: "));
         wprintf("<input type=\"text\" name=\"script_name\"><br />\n"
         	"<input type=\"submit\" name=\"create_button\" value=\"%s\">"
 		"</form></center>\n", _("Create"));
 
-	do_template("endbox");
+	do_template("endbox", NULL);
 
 	svput("BOXTITLE", WCS_STRING, _("Edit scripts"));
-	do_template("beginbox");
+	do_template("beginboxx", NULL);
 	wprintf("<br /><div align=center><a href=\"display_sieve\">%s</a><br /><br />\n",
 		_("Return to the script editing screen")
 	);
-	do_template("endbox");
+	do_template("endbox", NULL);
 
 	wprintf("</td><td>");
 
 	svput("BOXTITLE", WCS_STRING, _("Delete scripts"));
-	do_template("beginbox");
+	do_template("beginboxx", NULL);
 
 	wprintf(_("To delete an existing script, select the script "
 		"name from the list and click 'Delete'."));
@@ -682,7 +689,7 @@ void display_add_remove_scripts(char *message)
 	
         wprintf("<center>"
 		"<form method=\"POST\" action=\"delete_script\">\n");
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%ld\">\n", WC->nonce);
+	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
         wprintf("<select name=\"script_name\" size=10 style=\"width:100%%\">\n");
 
         serv_puts("MSIV listscripts");
@@ -702,7 +709,7 @@ void display_add_remove_scripts(char *message)
         wprintf("<input type=\"submit\" name=\"delete_button\" value=\"%s\" "
 		"onClick=\"return confirm('%s');\">", _("Delete script"), _("Delete this script?"));
         wprintf("</form></center>\n");
-	do_template("endbox");
+	do_template("endbox", NULL);
 
 	wprintf("</td></tr></table>\n");
 
@@ -983,7 +990,7 @@ void display_rules_editor_inner_div(void) {
 
 		wprintf("<td width=20%%>%s ", _("If") );
 
-		char *hfield_values[14][2] = {
+		char *hfield_values[15][2] = {
 			{	"from",		_("From")		},
 			{	"tocc",		_("To or Cc")		},
 			{	"subject",	_("Subject")		},
@@ -996,13 +1003,14 @@ void display_rules_editor_inner_div(void) {
 			{	"xmailer",	_("X-Mailer")		},
 			{	"xspamflag",	_("X-Spam-Flag")	},
 			{	"xspamstatus",	_("X-Spam-Status")	},
+			{	"listid",	_("List-ID")		},
 			{	"size",		_("Message size")	},
 			{	"all",		_("All")		}
 		};
 
 		wprintf("<select id=\"hfield%d\" name=\"hfield%d\" size=1 onChange=\"UpdateRules();\">",
 			i, i);
-		for (j=0; j<14; ++j) {
+		for (j=0; j<15; ++j) {
 			wprintf("<option %s value=\"%s\">%s</option>",
 				( (!strcasecmp(hfield, hfield_values[j][0])) ? "selected" : ""),
 				hfield_values[j][0],
@@ -1155,3 +1163,16 @@ void display_rules_editor_inner_div(void) {
 
 	free(rooms);
 }
+
+void _display_add_remove_scripts(void) {display_add_remove_scripts(NULL);}
+
+void 
+InitModule_SIEVE
+(void)
+{
+	WebcitAddUrlHandler(HKEY("display_sieve"), display_sieve, 0);
+	WebcitAddUrlHandler(HKEY("save_sieve"), save_sieve, 0);
+	WebcitAddUrlHandler(HKEY("display_add_remove_scripts"), _display_add_remove_scripts, 0);
+	WebcitAddUrlHandler(HKEY("create_script"), create_script, 0);
+	WebcitAddUrlHandler(HKEY("delete_script"), delete_script, 0);
+}
