@@ -1,5 +1,5 @@
 /* 
- * $Id: smtpqueue.c 6383 2008-06-16 02:16:02Z ajc $
+ * $Id: smtpqueue.c 7369 2009-04-18 12:27:01Z dothebart $
  *
  * Display the outbound SMTP queue
  */
@@ -83,14 +83,14 @@ void display_queue_msg(long msgnum)
 			}
 
 			if (!strcasecmp(keyword, "bounceto")) {
+				char *atsign;
 				extract_token(sender, buf, 1, '|', sizeof sender);
 
 				/* Strip off local hostname if it's our own */
-				char *atsign;
 				atsign = strchr(sender, '@');
 				if (atsign != NULL) {
 					++atsign;
-					if (!strcasecmp(atsign, serv_info.serv_nodename)) {
+					if (!strcasecmp(atsign, ChrPtr(WC->serv_info->serv_nodename))) {
 						--atsign;
 						*atsign = 0;
 					}
@@ -137,7 +137,7 @@ void display_queue_msg(long msgnum)
 
 	wprintf("</td><td>");
 	if (submitted > 0) {
-		webcit_fmt_date(buf, submitted, 1);
+		webcit_fmt_date(buf, 1024, submitted, 1);
 		wprintf("%s", buf);
 	}
 	else {
@@ -146,7 +146,7 @@ void display_queue_msg(long msgnum)
 
 	wprintf("</td><td>");
 	if (last_attempt > 0) {
-		webcit_fmt_date(buf, last_attempt, 1);
+		webcit_fmt_date(buf, 1024, last_attempt, 1);
 		wprintf("%s", buf);
 	}
 	else {
@@ -164,14 +164,19 @@ void display_queue_msg(long msgnum)
 
 
 void display_smtpqueue_inner_div(void) {
+	message_summary *Msg;
+	wcsession *WCC = WC;
 	int i;
 	int num_msgs;
+	StrBuf *Buf;
 
 	/* Check to see if we can go to the __CitadelSMTPspoolout__ room.
 	 * If not, we don't have access to the queue.
 	 */
-	gotoroom("__CitadelSMTPspoolout__");
-	if (!strcasecmp(WC->wc_roomname, "__CitadelSMTPspoolout__")) {
+	Buf = NewStrBufPlain(HKEY("__CitadelSMTPspoolout__"));
+	gotoroom(Buf);
+	FreeStrBuf(&Buf);
+	if (!strcasecmp(ChrPtr(WCC->wc_roomname), "__CitadelSMTPspoolout__")) {
 
 		num_msgs = load_msg_ptrs("MSGS ALL", 0);
 		if (num_msgs > 0) {
@@ -192,7 +197,9 @@ void display_smtpqueue_inner_div(void) {
 			wprintf("</i></b></td></tr>\n");
 
 			for (i=0; i<num_msgs; ++i) {
-				display_queue_msg(WC->msgarr[i]);
+				Msg = GetMessagePtrAt(i, WCC->summ);
+
+				display_queue_msg((Msg==NULL)? 0 : Msg->msgnum);
 			}
 
 			wprintf("</table>");
@@ -265,7 +272,12 @@ void display_smtpqueue(void)
 
 }
 
-
-
+void 
+InitModule_SMTP_QUEUE
+(void)
+{
+	WebcitAddUrlHandler(HKEY("display_smtpqueue"), display_smtpqueue, 0);
+	WebcitAddUrlHandler(HKEY("display_smtpqueue_inner_div"), display_smtpqueue_inner_div, 0);
+}
 
 /*@}*/
