@@ -1,35 +1,31 @@
 /*
- * $Id: tcp_sockets.c 6088 2008-03-02 14:20:45Z dothebart $
+ * $Id: tcp_sockets.c 7142 2009-02-21 14:11:58Z dothebart $
  */
-/** 
- * \defgroup TcpSockets TCP client socket module for WebCit
- * \ingroup CitadelCommunitacion
- */
-/*@{*/
 
 /*
  * Uncomment this to log all communications with the Citadel server
 #define SERV_TRACE 1
  */
 
+
 #include "webcit.h"
 #include "webserver.h"
 
-/**
- * \brief register the timeout
- * \param signum signalhandler number
+/*
+ *  register the timeout
+ *  signum signalhandler number
  * \return signals
  */
 RETSIGTYPE timeout(int signum)
 {
-	lprintf(1, "Connection timed out.\n");
-	exit(3);
+	lprintf(1, "Connection timed out; unable to reach citserver\n");
+	/* no exit here, since we need to server the connection unreachable thing. exit(3); */
 }
 
 
-/**
- * \brief Connect a unix domain socket
- * \param sockpath where to open a unix domain socket
+/*
+ *  Connect a unix domain socket
+ *  sockpath where to open a unix domain socket
  */
 int uds_connectsock(char *sockpath)
 {
@@ -58,10 +54,10 @@ int uds_connectsock(char *sockpath)
 }
 
 
-/**
- * \brief Connect a TCP/IP socket
- * \param host the host to connect to
- * \param service the service on the host to call
+/*
+ *  Connect a TCP/IP socket
+ *  host the host to connect to
+ *  service the service on the host to call
  */
 int tcp_connectsock(char *host, char *service)
 {
@@ -118,17 +114,17 @@ int tcp_connectsock(char *host, char *service)
 
 
 
-/**
- * \brief Input binary data from socket
- * \param buf the buffer to get the input to
- * \param bytes the maximal number of bytes to read
+/*
+ *  Input binary data from socket
+ *  buf the buffer to get the input to
+ *  bytes the maximal number of bytes to read
  */
-inline void _serv_read(char *buf, int bytes, struct wcsession *WCC)
+inline void _serv_read(char *buf, int bytes, wcsession *WCC)
 {
 	int len, rlen;
 
 	len = 0;
-	while (len < bytes) {
+	while ((len < bytes) && (WCC->serv_sock != -1)){
 		rlen = read(WCC->serv_sock, &buf[len], bytes - len);
 		if (rlen < 1) {
 			lprintf(1, "Server connection broken: %s\n",
@@ -147,16 +143,16 @@ inline void _serv_read(char *buf, int bytes, struct wcsession *WCC)
 
 void serv_read(char *buf, int bytes)
 {
-	struct wcsession *WCC = WC;
+	wcsession *WCC = WC;
 	_serv_read(buf, bytes, WCC);
 }
 
-/**
- * \brief input string from pipe
+/*
+ *  input string from pipe
  */
 int serv_getln(char *strbuf, int bufsize)
 {
-	struct wcsession *WCC = WC;
+	wcsession *WCC = WC;
 	int ch, len;
 	char buf[2];
 
@@ -168,7 +164,7 @@ int serv_getln(char *strbuf, int bufsize)
 		if ((ch != 13) && (ch != 10)) {
 			strbuf[len++] = ch;
 		}
-	} while ((ch != 10) && (ch != 0) && (len < (bufsize-1)));
+	} while ((ch != 10) && (ch != 0) && (len < (bufsize-1)) && (WCC->serv_sock != -1));
 	strbuf[len] = 0;
 #ifdef SERV_TRACE
 	lprintf(9, "%3d>%s\n", WC->serv_sock, strbuf);
@@ -176,14 +172,71 @@ int serv_getln(char *strbuf, int bufsize)
 	return len;
 }
 
+int StrBuf_ServGetln(StrBuf *buf)
+{
+	const char *ErrStr;
+	int rc;
 
+	rc = StrBufTCP_read_line(buf, &WC->serv_sock, 0, &ErrStr);
+	if (rc < 0)
+	{
+		lprintf(1, "Server connection broken: %s\n",
+			ErrStr);
+		wc_backtrace();
+		WC->serv_sock = (-1);
+		WC->connected = 0;
+		WC->logged_in = 0;
+	}
+	return rc;
+}
 
-/**
- * \brief send binary to server
- * \param buf the buffer to write to citadel server
- * \param nbytes how many bytes to send to citadel server
+int StrBuf_ServGetlnBuffered(StrBuf *buf)
+{
+	wcsession *WCC = WC;
+	const char *ErrStr;
+	int rc;
+
+	rc = StrBufTCP_read_buffered_line(buf, 
+					  WCC->ReadBuf, 
+					  &WCC->serv_sock, 
+					  5, 1, 
+					  &ErrStr);
+	if (rc < 0)
+	{
+		lprintf(1, "Server connection broken: %s\n",
+			ErrStr);
+		wc_backtrace();
+		WCC->serv_sock = (-1);
+		WCC->connected = 0;
+		WCC->logged_in = 0;
+	}
+	return rc;
+}
+
+int StrBuf_ServGetBLOB(StrBuf *buf, long BlobSize)
+{
+	const char *Err;
+	int rc;
+	
+	rc = StrBufReadBLOB(buf, &WC->serv_sock, 1, BlobSize, &Err);
+	if (rc < 0)
+	{
+		lprintf(1, "Server connection broken: %s\n",
+			Err);
+		wc_backtrace();
+		WC->serv_sock = (-1);
+		WC->connected = 0;
+		WC->logged_in = 0;
+	}
+	return rc;
+}
+
+/*
+ *  send binary to server
+ *  buf the buffer to write to citadel server
+ *  nbytes how many bytes to send to citadel server
  */
-void serv_write(char *buf, int nbytes)
+void serv_write(const char *buf, int nbytes)
 {
 	int bytes_written = 0;
 	int retval;
@@ -204,11 +257,11 @@ void serv_write(char *buf, int nbytes)
 }
 
 
-/**
- * \brief send line to server
- * \param string the line to send to the citadel server
+/*
+ *  send line to server
+ *  string the line to send to the citadel server
  */
-void serv_puts(char *string)
+void serv_puts(const char *string)
 {
 #ifdef SERV_TRACE
 	lprintf(9, "%3d<%s\n", WC->serv_sock, string);
@@ -217,11 +270,24 @@ void serv_puts(char *string)
 	serv_write("\n", 1);
 }
 
+/*
+ *  send line to server
+ *  string the line to send to the citadel server
+ */
+void serv_putbuf(const StrBuf *string)
+{
+#ifdef SERV_TRACE
+	lprintf(9, "%3d<%s\n", WC->serv_sock, ChrPtr(string));
+#endif
+	serv_write(ChrPtr(string), StrLength(string));
+	serv_write("\n", 1);
+}
 
-/**
- * \brief convenience function to send stuff to the server
- * \param format the formatstring
- * \param ... the entities to insert into format 
+
+/*
+ *  convenience function to send stuff to the server
+ *  format the formatstring
+ *  ... the entities to insert into format 
  */
 void serv_printf(const char *format,...)
 {
@@ -242,5 +308,3 @@ void serv_printf(const char *format,...)
 #endif
 }
 
-
-/*@}*/
