@@ -1,10 +1,13 @@
 /*
- * $Id: webserver.c 6301 2008-05-20 06:49:02Z dothebart $
+ * $Id: webserver.c 7336 2009-04-11 22:26:39Z dothebart $
  *
  * This contains a simple multithreaded TCP server manager.  It sits around
  * waiting on the specified port for incoming HTTP connections.  When a
  * connection is established, it calls context_loop() from context_loop.c.
  *
+ * Copyright (c) 1996-2009 by the citadel.org developers.
+ * This program is released under the terms of the GNU General Public License v3.
+ *
  */
 
 #include "webcit.h"
@@ -13,7 +16,7 @@
 #if HAVE_BACKTRACE
 #include <execinfo.h>
 #endif
-
+#include "modules_init.h"
 #ifndef HAVE_SNPRINTF
 int vsnprintf(char *buf, size_t max, const char *fmt, va_list argp);
 #endif
@@ -24,7 +27,8 @@ int is_https = 0;		/* Nonzero if I am an HTTPS service */
 int follow_xff = 0;		/* Follow X-Forwarded-For: header */
 int home_specified = 0;		/* did the user specify a homedir? */
 int time_to_die = 0;            /* Nonzero if server is shutting down */
-extern void *context_loop(int);
+int DisableGzip = 0;
+extern void *context_loop(int*);
 extern void *housekeeping_loop(void);
 extern pthread_mutex_t SessionListMutex;
 extern pthread_key_t MyConKey;
@@ -110,10 +114,12 @@ int ig_tcp_server(char *ip_addr, int port_number, int queue_len)
 	i = 1;
 	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));
 
+	#ifndef __APPLE__
 	fcntl(s, F_SETFL, O_NONBLOCK); /* maide: this statement is incorrect
 					  there should be a preceding F_GETFL
 					  and a bitwise OR with the previous
 					  fd flags */
+	#endif
 	
 	if (bind(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
 		lprintf(1, "Can't bind: %s\n", strerror(errno));
@@ -144,7 +150,7 @@ int ig_uds_server(char *sockpath, int queue_len)
 	if (actual_queue_len < 5) actual_queue_len = 5;
 
 	i = unlink(sockpath);
-	if (i != 0) if (errno != ENOENT) {
+	if ((i != 0) && (errno != ENOENT)) {
 		lprintf(1, "webcit: can't unlink %s: %s\n",
 			sockpath, strerror(errno));
 		exit(WC_EXIT_BIND);
@@ -181,50 +187,56 @@ int ig_uds_server(char *sockpath, int queue_len)
 
 
 /*
- * \brief Read data from the client socket.
- * \param sock socket fd to read from
- * \param buf buffer to read into 
- * \param bytes number of bytes to read
- * \param timeout Number of seconds to wait before timing out
- * \return values are\
- *      1       Requested number of bytes has been read.\
- *      0       Request timed out.\
- *	   -1   	Connection is broken, or other error.
+ * Read data from the client socket.
+ *
+ * sock		socket fd to read from
+ * buf		buffer to read into 
+ * bytes	number of bytes to read
+ * timeout	Number of seconds to wait before timing out
+ *
+ * Possible return values:
+ *      1       Requested number of bytes has been read.
+ *      0       Request timed out.
+ *	-1   	Connection is broken, or other error.
  */
-int client_read_to(int sock, char *buf, int bytes, int timeout)
+int client_read_to(int *sock, StrBuf *Target, StrBuf *Buf, int bytes, int timeout)
 {
-	int len, rlen;
-	fd_set rfds;
-	struct timeval tv;
-	int retval;
-
+	const char *Error;
+	int retval = 0;
 
 #ifdef HAVE_OPENSSL
 	if (is_https) {
-		return (client_read_ssl(buf, bytes, timeout));
+		while ((StrLength(Buf) + StrLength(Target) < bytes) &&
+		       (retval >= 0))
+			retval = client_read_sslbuffer(Buf, timeout);
+		if (retval >= 0) {
+			StrBufAppendBuf(Target, Buf, 0); /* todo: Buf > bytes? */
+#ifdef HTTP_TRACING
+			write(2, "\033[32m", 5);
+			write(2, buf, bytes);
+			write(2, "\033[30m", 5);
+#endif
+			return 1;
+		}
+		else {
+			lprintf(2, "client_read_ssl() failed\n");
+			return -1;
+		}
 	}
 #endif
 
-	len = 0;
-	while (len < bytes) {
-		FD_ZERO(&rfds);
-		FD_SET(sock, &rfds);
-		tv.tv_sec = timeout;
-		tv.tv_usec = 0;
-
-		retval = select((sock) + 1, &rfds, NULL, NULL, &tv);
-		if (FD_ISSET(sock, &rfds) == 0) {
-			return (0);
-		}
-
-		rlen = read(sock, &buf[len], bytes - len);
-
-		if (rlen < 1) {
-			lprintf(2, "client_read() failed: %s\n",
-				strerror(errno));
-			return (-1);
-		}
-		len = len + rlen;
+	if (StrLength(Buf) > 0) {/*/// todo: what if Buf > bytes?*/
+		StrBufAppendBuf(Target, Buf, 0);
+	}
+	retval = StrBufReadBLOB(Target, 
+			   sock, 
+			   (StrLength(Target) > 0), 
+			   bytes - StrLength(Target), 
+				&Error);
+	if (retval < 0) {
+		lprintf(2, "client_read() failed: %s\n",
+			Error);
+		return retval;
 	}
 
 #ifdef HTTP_TRACING
@@ -232,263 +244,141 @@ int client_read_to(int sock, char *buf, int bytes, int timeout)
 	write(2, buf, bytes);
 	write(2, "\033[30m", 5);
 #endif
-	return (1);
+	return 1;
 }
 
+
 /*
- * \brief write data to the client
- * \param buf data to write to the client
- * \param count size of buffer
+ * Begin buffering HTTP output so we can transmit it all in one write operation later.
  */
-ssize_t client_write(const void *buf, size_t count)
+void begin_burst(void)
 {
-        char *newptr;
-        size_t newalloc;
-        size_t bytesWritten = 0;
-        ssize_t res;
+	if (WC->WBuf == NULL) {
+		WC->WBuf = NewStrBufPlain(NULL, 32768);
+	}
+}
+
+
+/*
+ * Finish buffering HTTP output.  [Compress using zlib and] output with a Content-Length: header.
+ */
+long end_burst(void)
+{
+	wcsession *WCC = WC;
+        const char *ptr, *eptr;
+        long count;
+	ssize_t res;
         fd_set wset;
         int fdflags;
 
-	if (WC->burst != NULL) {
-		if ((WC->burst_len + count) >= WC->burst_alloc) {
-			newalloc = (WC->burst_alloc * 2);
-			if ((WC->burst_len + count) >= newalloc) {
-				newalloc += count;
-			}
-			newptr = realloc(WC->burst, newalloc);
-			if (newptr != NULL) {
-				WC->burst = newptr;
-				WC->burst_alloc = newalloc;
-			}
-		}
-		if ((WC->burst_len + count) < WC->burst_alloc) {
-			memcpy(&WC->burst[WC->burst_len], buf, count);
-			WC->burst_len += count;
-			return (count);
-		}
-		else {
-			return(-1);
-		}
+	//#ifdef HAVE_ZLIB
+	/* Perform gzip compression, if enabled and supported by client */
+	if (!DisableGzip && (WCC->gzip_ok) && CompressBuffer(WCC->WBuf))
+	{
+		hprintf("Content-encoding: gzip\r\n");
 	}
+	//#endif	/* HAVE_ZLIB */
+
+	hprintf("Content-length: %d\r\n\r\n", StrLength(WCC->WBuf));
+
+	ptr = ChrPtr(WCC->HBuf);
+	count = StrLength(WCC->HBuf);
+	eptr = ptr + count;
+
 #ifdef HAVE_OPENSSL
 	if (is_https) {
-		client_write_ssl((char *) buf, count);
+		client_write_ssl(WCC->HBuf);
+		client_write_ssl(WCC->WBuf);
 		return (count);
 	}
 #endif
+
+	
 #ifdef HTTP_TRACING
+	
 	write(2, "\033[34m", 5);
-	write(2, buf, count);
+	write(2, ptr, StrLength(WCC->WBuf));
 	write(2, "\033[30m", 5);
 #endif
 	fdflags = fcntl(WC->http_sock, F_GETFL);
 
-        while (bytesWritten < count) {
+	while (ptr < eptr) {
                 if ((fdflags & O_NONBLOCK) == O_NONBLOCK) {
                         FD_ZERO(&wset);
-                        FD_SET(WC->http_sock, &wset);
-                        if (select(WC->http_sock + 1, NULL, &wset, NULL, NULL) == -1) {
+                        FD_SET(WCC->http_sock, &wset);
+                        if (select(WCC->http_sock + 1, NULL, &wset, NULL, NULL) == -1) {
                                 lprintf(2, "client_write: Socket select failed (%s)\n", strerror(errno));
                                 return -1;
                         }
                 }
 
-                if ((res = write(WC->http_sock, (char*)buf + bytesWritten,
-                  count - bytesWritten)) == -1) {
+                if ((res = write(WCC->http_sock, 
+				 ptr,
+				 count)) == -1) {
                         lprintf(2, "client_write: Socket write failed (%s)\n", strerror(errno));
+		        wc_backtrace();
                         return res;
                 }
-                bytesWritten += res;
+                count -= res;
+		ptr += res;
         }
 
-	return bytesWritten;
-}
+	ptr = ChrPtr(WCC->WBuf);
+	count = StrLength(WCC->WBuf);
+	eptr = ptr + count;
 
-/*
- * \brief Begin buffering HTTP output so we can transmit it all in one write operation later.
- */
-void begin_burst(void)
-{
-	if (WC->burst != NULL) {
-		free(WC->burst);
-		WC->burst = NULL;
-	}
-	WC->burst_len = 0;
-	WC->burst_alloc = 32768;
-	WC->burst = malloc(WC->burst_alloc);
-}
-
-
-/*
- * \brief uses the same calling syntax as compress2(), but it
- * creates a stream compatible with HTTP "Content-encoding: gzip"
- */
-#ifdef HAVE_ZLIB
-#define DEF_MEM_LEVEL 8 /*< memlevel??? */
-#define OS_CODE 0x03	/*< unix */
-int ZEXPORT compress_gzip(Bytef * dest,         /*< compressed buffer*/
-			  size_t * destLen,     /*< length of the compresed data */
-			  const Bytef * source, /*< source to encode */
-			  uLong sourceLen,      /*< length of source to encode */
-			  int level)            /*< compression level */
-{
-	const int gz_magic[2] = { 0x1f, 0x8b };	/* gzip magic header */
-
-	/* write gzip header */
-	snprintf((char *) dest, *destLen, 
-		 "%c%c%c%c%c%c%c%c%c%c",
-		 gz_magic[0], gz_magic[1], Z_DEFLATED,
-		 0 /*flags */ , 0, 0, 0, 0 /*time */ , 0 /* xflags */ ,
-		 OS_CODE);
-
-	/* normal deflate */
-	z_stream stream;
-	int err;
-	stream.next_in = (Bytef *) source;
-	stream.avail_in = (uInt) sourceLen;
-	stream.next_out = dest + 10L;	// after header
-	stream.avail_out = (uInt) * destLen;
-	if ((uLong) stream.avail_out != *destLen)
-		return Z_BUF_ERROR;
-
-	stream.zalloc = (alloc_func) 0;
-	stream.zfree = (free_func) 0;
-	stream.opaque = (voidpf) 0;
-
-	err = deflateInit2(&stream, level, Z_DEFLATED, -MAX_WBITS,
-			   DEF_MEM_LEVEL, Z_DEFAULT_STRATEGY);
-	if (err != Z_OK)
-		return err;
-
-	err = deflate(&stream, Z_FINISH);
-	if (err != Z_STREAM_END) {
-		deflateEnd(&stream);
-		return err == Z_OK ? Z_BUF_ERROR : err;
-	}
-	*destLen = stream.total_out + 10L;
-
-	/* write CRC and Length */
-	uLong crc = crc32(0L, source, sourceLen);
-	int n;
-	for (n = 0; n < 4; ++n, ++*destLen) {
-		dest[*destLen] = (int) (crc & 0xff);
-		crc >>= 8;
-	}
-	uLong len = stream.total_in;
-	for (n = 0; n < 4; ++n, ++*destLen) {
-		dest[*destLen] = (int) (len & 0xff);
-		len >>= 8;
-	}
-	err = deflateEnd(&stream);
-	return err;
-}
+#ifdef HTTP_TRACING
+	
+	write(2, "\033[34m", 5);
+	write(2, ptr, StrLength(WCC->WBuf));
+	write(2, "\033[30m", 5);
 #endif
 
-/*
- * \brief Finish buffering HTTP output.  [Compress using zlib and] output with a Content-Length: header.
- */
-void end_burst(void)
-{
-	size_t the_len;
-	char *the_data;
+        while (ptr < eptr) {
+                if ((fdflags & O_NONBLOCK) == O_NONBLOCK) {
+                        FD_ZERO(&wset);
+                        FD_SET(WCC->http_sock, &wset);
+                        if (select(WCC->http_sock + 1, NULL, &wset, NULL, NULL) == -1) {
+                                lprintf(2, "client_write: Socket select failed (%s)\n", strerror(errno));
+                                return -1;
+                        }
+                }
 
-	if (WC->burst == NULL)
-		return;
+                if ((res = write(WCC->http_sock, 
+				 ptr,
+				 count)) == -1) {
+                        lprintf(2, "client_write: Socket write failed (%s)\n", strerror(errno));
+			wc_backtrace();
+                        return res;
+                }
+                count -= res;
+		ptr += res;
+        }
 
-	the_len = WC->burst_len;
-	the_data = WC->burst;
-
-	WC->burst_len = 0;
-	WC->burst_alloc = 0;
-	WC->burst = NULL;
-
-#ifdef HAVE_ZLIB
-	/* Perform gzip compression, if enabled and supported by client */
-	if (WC->gzip_ok) {
-		char *compressed_data = NULL;
-		size_t compressed_len;
-
-		compressed_len = ((the_len * 101) / 100) + 100;
-		compressed_data = malloc(compressed_len);
-
-		if (compress_gzip((Bytef *) compressed_data,
-				  &compressed_len,
-				  (Bytef *) the_data,
-				  (uLongf) the_len, Z_BEST_SPEED) == Z_OK) {
-			wprintf("Content-encoding: gzip\r\n");
-			free(the_data);
-			the_data = compressed_data;
-			the_len = compressed_len;
-		} else {
-			free(compressed_data);
-		}
-	}
-#endif	/* HAVE_ZLIB */
-
-	wprintf("Content-length: %d\r\n\r\n", the_len);
-	client_write(the_data, the_len);
-	free(the_data);
-	return;
+	return StrLength(WCC->WBuf);
 }
 
 
 
 /*
- * \brief Read data from the client socket with default timeout.
+ * Read data from the client socket with default timeout.
  * (This is implemented in terms of client_read_to() and could be
  * justifiably moved out of sysdep.c)
- * \param sock the socket fd to read from
- * \param buf the buffer to write to
- * \param bytes Number of bytes to read
+ *
+ * sock		the socket fd to read from
+ * buf		the buffer to write to
+ * bytes	Number of bytes to read
  */
-int client_read(int sock, char *buf, int bytes)
+int client_read(int *sock, StrBuf *Target, StrBuf *buf, int bytes)
 {
-	return (client_read_to(sock, buf, bytes, SLEEPING));
+	return (client_read_to(sock, Target, buf, bytes, SLEEPING));
 }
 
 
-/*
- * \brief Get a LF-terminated line of text from the client.
- * (This is implemented in terms of client_read() and could be
- * justifiably moved out of sysdep.c)
- * \param sock socket fd to get client line from
- * \param buf buffer to write read data to
- * \param bufsiz how many bytes to read
- * \return  number of bytes read???
- */
-int client_getln(int sock, char *buf, int bufsiz)
-{
-	int i, retval;
-
-	/* Read one character at a time.*/
-	for (i = 0;; i++) {
-		retval = client_read(sock, &buf[i], 1);
-		if (retval != 1 || buf[i] == '\n' || i == (bufsiz-1))
-			break;
-		if ( (!isspace(buf[i])) && (!isprint(buf[i])) ) {
-			/* Non printable character recieved from client */
-			return(-1);
-		}
-	}
-
-	/* If we got a long line, discard characters until the newline. */
-	if (i == (bufsiz-1))
-		while (buf[i] != '\n' && retval == 1)
-			retval = client_read(sock, &buf[i], 1);
-
-	/*
-	 * Strip any trailing non-printable characters.
-	 */
-	buf[i] = 0;
-	while ((i > 0) && (!isprint(buf[i - 1]))) {
-		buf[--i] = 0;
-	}
-	return (retval);
-}
 
 /*
- * \brief shut us down the regular way.
- * param signum the signal we want to forward
+ * Shut us down the regular way.
+ * signum is the signal we want to forward
  */
 pid_t current_child;
 void graceful_shutdown_watcher(int signum) {
@@ -498,13 +388,77 @@ void graceful_shutdown_watcher(int signum) {
 		exit(0);
 }
 
+
+int ClientGetLine(int *sock, StrBuf *Target, StrBuf *CLineBuf)
+{
+	const char *Error, *pch, *pchs;
+	int rlen, len, retval = 0;
+
+#ifdef HAVE_OPENSSL
+	if (is_https) {
+		int ntries = 0;
+		if (StrLength(CLineBuf) > 0) {
+			pchs = ChrPtr(CLineBuf);
+			pch = strchr(pchs, '\n');
+			if (pch != NULL) {
+				rlen = 0;
+				len = pch - pchs;
+				if (len > 0 && (*(pch - 1) == '\r') )
+					rlen ++;
+				StrBufSub(Target, CLineBuf, 0, len - rlen);
+				StrBufCutLeft(CLineBuf, len + 1);
+				return len - rlen;
+			}
+		}
+
+		while (retval == 0) { 
+				pch = NULL;
+				pchs = ChrPtr(CLineBuf);
+				if (*pchs != '\0')
+					pch = strchr(pchs, '\n');
+				if (pch == NULL) {
+					retval = client_read_sslbuffer(CLineBuf, SLEEPING);
+					pchs = ChrPtr(CLineBuf);
+					pch = strchr(pchs, '\n');
+				}
+				if (retval == 0) {
+					sleeeeeeeeeep(1);
+					ntries ++;
+				}
+				if (ntries > 10)
+					return 0;
+		}
+		if ((retval > 0) && (pch != NULL)) {
+			rlen = 0;
+			len = pch - pchs;
+			if (len > 0 && (*(pch - 1) == '\r') )
+				rlen ++;
+			StrBufSub(Target, CLineBuf, 0, len - rlen);
+			StrBufCutLeft(CLineBuf, len + 1);
+			return len - rlen;
+
+		}
+		else 
+			return -1;
+	}
+	else 
+#endif
+		return StrBufTCP_read_buffered_line(Target, 
+						    CLineBuf,
+						    sock,
+						    5,
+						    1,
+						    &Error);
+}
+
+
+
 /*
- * \brief shut us down the regular way.
- * param signum the signal we want to forward
+ * Shut us down the regular way.
+ * signum is the signal we want to forward
  */
 pid_t current_child;
 void graceful_shutdown(int signum) {
-//	kill(current_child, signum);
 	char wd[SIZ];
 	FILE *FD;
 	int fd;
@@ -520,12 +474,6 @@ void graceful_shutdown(int signum) {
 }
 
 
-/*
- * \brief	Start running as a daemon.  
- *
- * param	do_close_stdio		Only close stdio if set.
- */
-
 /*
  * Start running as a daemon.
  */
@@ -571,16 +519,12 @@ void start_daemon(char *pid_file)
 			exit(errno);
 		}
 	
-		else if (current_child == 0) {	// child process
-//			signal(SIGTERM, graceful_shutdown);
+		else if (current_child == 0) {	/* child process */
 			signal(SIGHUP, graceful_shutdown);
 
 			return; /* continue starting webcit. */
 		}
-	
-		else { // watcher process
-//			signal(SIGTERM, SIG_IGN);
-//			signal(SIGHUP, SIG_IGN);
+		else { /* watcher process */
 			if (pid_file) {
 				fp = fopen(pid_file, "w");
 				if (fp != NULL) {
@@ -627,12 +571,12 @@ void start_daemon(char *pid_file)
 }
 
 /*
- * \brief	Spawn an additional worker thread into the pool.
+ * Spawn an additional worker thread into the pool.
  */
 void spawn_another_worker_thread()
 {
-	pthread_t SessThread;	/*< Thread descriptor */
-	pthread_attr_t attr;	/*< Thread attributes */
+	pthread_t SessThread;	/* Thread descriptor */
+	pthread_attr_t attr;	/* Thread attributes */
 	int ret;
 
 	lprintf(3, "Creating a new thread\n");
@@ -663,16 +607,27 @@ void spawn_another_worker_thread()
 	pthread_attr_destroy(&attr);
 }
 
+/* #define DBG_PRINNT_HOOKS_AT_START */
+#ifdef DBG_PRINNT_HOOKS_AT_START
+const char foobuf[32];
+const char *nix(void *vptr) {snprintf(foobuf, 32, "%0x", (long) vptr); return foobuf;}
+#endif 
+extern int dbg_analyze_msg;
+extern int dbg_bactrace_template_errors;
+extern int DumpTemplateI18NStrings;
+extern StrBuf *I18nDump;
+void InitTemplateCache(void);
+extern int LoadTemplates;
+extern void LoadZoneFiles(void);
+StrBuf *csslocal = NULL;
 /*
- * \brief Here's where it all begins.
- * \param argc number of commandline args
- * \param argv the commandline arguments
+ * Here's where it all begins.
  */
 int main(int argc, char **argv)
 {
-	pthread_t SessThread;	/*< Thread descriptor */
-	pthread_attr_t attr;	/*< Thread attributes */
-	int a, i;	        	/*< General-purpose variables */
+	pthread_t SessThread;		/* Thread descriptor */
+	pthread_attr_t attr;		/* Thread attributes */
+	int a, i;	        	/* General-purpose variables */
 	char tracefile[PATH_MAX];
 	char ip_addr[256]="0.0.0.0";
 	char dirbuffer[PATH_MAX]="";
@@ -688,9 +643,30 @@ int main(int argc, char **argv)
 	char *locale = NULL;
 	char *mo = NULL;
 #endif /* ENABLE_NLS */
-	char uds_listen_path[PATH_MAX];	/*< listen on a unix domain socket? */
+	char uds_listen_path[PATH_MAX];	/* listen on a unix domain socket? */
+	const char *I18nDumpFile;
+
+	WildFireInitBacktrace(argv[0], 2);
+
+	HandlerHash = NewHash(1, NULL);
+	PreferenceHooks = NewHash(1, NULL);
+	WirelessTemplateCache = NewHash(1, NULL);
+	WirelessLocalTemplateCache = NewHash(1, NULL);
+	LocalTemplateCache = NewHash(1, NULL);
+	TemplateCache = NewHash(1, NULL);
+	GlobalNS = NewHash(1, NULL);
+	Iterators = NewHash(1, NULL);
+	Conditionals = NewHash(1, NULL);
+	MsgHeaderHandler = NewHash(1, NULL);
+	MimeRenderHandler = NewHash(1, NULL);
+	SortHash = NewHash(1, NULL);
+
+	LoadZoneFiles();
+
+#ifdef DBG_PRINNT_HOOKS_AT_START
+	dbg_PrintHash(HandlerHash, nix, NULL);
+#endif
 
-	HandlerHash = NewHash (1, NULL);
 	/* Ensure that we are linked to the correct version of libcitadel */
 	if (libcitadel_version_number() < LIBCITADEL_VERSION_NUMBER) {
 		fprintf(stderr, " You are running libcitadel version %d.%02d\n",
@@ -704,9 +680,9 @@ int main(int argc, char **argv)
 
 	/* Parse command line */
 #ifdef HAVE_OPENSSL
-	while ((a = getopt(argc, argv, "h:i:p:t:x:dD:cfs")) != EOF)
+	while ((a = getopt(argc, argv, "h:i:p:t:T:x:dD:G:cfsZ")) != EOF)
 #else
-	while ((a = getopt(argc, argv, "h:i:p:t:x:dD:cf")) != EOF)
+	while ((a = getopt(argc, argv, "h:i:p:t:T:x:dD:G:cfZ")) != EOF)
 #endif
 		switch (a) {
 		case 'h':
@@ -743,6 +719,14 @@ int main(int argc, char **argv)
 			freopen(tracefile, "w", stderr);
 			freopen(tracefile, "r", stdin);
 			break;
+		case 'T':
+			LoadTemplates = atoi(optarg);
+			dbg_analyze_msg = (LoadTemplates && (1<<1)) != 0;
+			dbg_bactrace_template_errors = (LoadTemplates && (1<<2)) != 0;
+			break;
+		case 'Z':
+			DisableGzip = 1;
+			break;
 		case 'x':
 			verbosity = atoi(optarg);
 			break;
@@ -767,11 +751,17 @@ int main(int argc, char **argv)
 		case 's':
 			is_https = 1;
 			break;
+		case 'G':
+			DumpTemplateI18NStrings = 1;
+			I18nDump = NewStrBufPlain(HKEY("int templatestrings(void)\n{\n"));
+			I18nDumpFile = optarg;
+			break;
 		default:
 			fprintf(stderr, "usage: webcit "
 				"[-i ip_addr] [-p http_port] "
 				"[-t tracefile] [-c] [-f] "
-				"[-d] "
+				"[-T Templatedebuglevel] "
+				"[-d] [-Z] [-G i18ndumpfile] "
 #ifdef HAVE_OPENSSL
 				"[-s] "
 #endif
@@ -786,17 +776,16 @@ int main(int argc, char **argv)
 	}
 
 	/* daemonize, if we were asked to */
-	if (running_as_daemon) {
+	if (!DumpTemplateI18NStrings && running_as_daemon) {
 		start_daemon(pidfile);
 	}
 	else {
-///		signal(SIGTERM, graceful_shutdown);
 		signal(SIGHUP, graceful_shutdown);
 	}
 
 	/* Tell 'em who's in da house */
 	lprintf(1, PACKAGE_STRING "\n");
-	lprintf(1, "Copyright (C) 1996-2008 by the Citadel development team.\n"
+	lprintf(1, "Copyright (C) 1996-2009 by the Citadel development team.\n"
 		"This software is distributed under the terms of the "
 		"GNU General Public License.\n\n"
 	);
@@ -806,6 +795,7 @@ int main(int argc, char **argv)
 #ifdef ENABLE_NLS
 	initialize_locales();
 
+
 	locale = setlocale(LC_ALL, "");
 
 	mo = malloc(strlen(webcitdir) + 20);
@@ -813,7 +803,6 @@ int main(int argc, char **argv)
 	free(mo);
 	lprintf(9, "Text domain: %s\n", textdomain("webcit"));
 	lprintf(9, "Text domain Charset: %s\n", bind_textdomain_codeset("webcit","UTF8"));
-	preset_locale();
 #endif
 
 
@@ -857,9 +846,43 @@ int main(int argc, char **argv)
 		perror("chdir");
 	}
 	LoadIconDir(static_icon_dir);
+
+	initialise_modules();
 	initialize_viewdefs();
 	initialize_axdefs();
 
+	InitTemplateCache();
+	if (DumpTemplateI18NStrings) {
+		FILE *fd;
+		StrBufAppendBufPlain(I18nDump, HKEY("}\n"), 0);
+	        if (StrLength(I18nDump) < 50) {
+			lprintf(1, "********************************************************************************\n");
+			lprintf(1, "*        No strings found in templates! are you shure they're there?           *\n");
+			lprintf(1, "********************************************************************************\n");
+			return -1;
+		}
+		fd = fopen(I18nDumpFile, "w");
+	        if (fd == NULL) {
+			lprintf(1, "********************************************************************************\n");
+			lprintf(1, "*                  unable to open I18N dumpfile [%s]         *\n", I18nDumpFile);
+			lprintf(1, "********************************************************************************\n");
+			return -1;
+		}
+		fwrite(ChrPtr(I18nDump), 1, StrLength(I18nDump), fd);
+		fclose(fd);
+		return 0;
+	}
+
+	if (!access("static.local/webcit.css", R_OK)) {
+		csslocal = NewStrBufPlain(HKEY("<link href=\"static.local/webcit.css\" rel=\"stylesheet\" type=\"text/css\">"));
+	}
+
+	/* Tell libical to return an error instead of aborting if it sees badly formed iCalendar data. */
+	icalerror_errors_are_fatal = 0;
+
+	/* Use our own prefix on tzid's generated from system tzdata */
+	icaltimezone_set_tzid_prefix("/citadel.org/");
+
 	/*
 	 * Set up a place to put thread-specific data.
 	 * We only need a single pointer per thread - it points to the
@@ -928,10 +951,41 @@ int main(int argc, char **argv)
 	/* now the original thread becomes another worker */
 	worker_entry();
 	ShutDownLibCitadel ();
+	DeleteHash(&HandlerHash);
+	DeleteHash(&PreferenceHooks);
 	return 0;
 }
 
 
+void ShutDownWebcit(void)
+{
+	DeleteHash(&ZoneHash);
+	free_zone_directory ();
+	icaltimezone_release_zone_tab ();
+	icalmemory_free_ring ();
+	ShutDownLibCitadel ();
+	DeleteHash(&HandlerHash);
+	DeleteHash(&PreferenceHooks);
+	DeleteHash(&GlobalNS);
+	DeleteHash(&WirelessTemplateCache);
+	DeleteHash(&WirelessLocalTemplateCache);
+	DeleteHash(&TemplateCache);
+	DeleteHash(&LocalTemplateCache);
+	DeleteHash(&Iterators);
+	DeleteHash(&MimeRenderHandler);
+	DeleteHash(&Conditionals);
+	DeleteHash(&MsgHeaderHandler);
+	DeleteHash(&SortHash);
+#ifdef ENABLE_NLS
+	ShutdownLocale();
+#endif
+#ifdef HAVE_OPENSSL
+	if (is_https) {
+		shutdown_ssl();
+	}
+#endif
+}
+
 /*
  * Entry point for worker threads
  */
@@ -965,19 +1019,19 @@ void worker_entry(void)
 			if (msock > 0)	ret = select(msock+1, &tempset, NULL, NULL,  &tv);
 			end_critical_section(S_SELECT);
 			if ((ret < 0) && (errno != EINTR) && (errno != EAGAIN))
-			{// EINTR and EAGAIN are thrown but not of interest.
+			{/* EINTR and EAGAIN are thrown but not of interest. */
 				lprintf(2, "accept() failed:%d %s\n",
 					errno, strerror(errno));
 			}
 			else if ((ret > 0) && (msock > 0) && FD_ISSET(msock, &tempset))
-			{// Successfully selected, and still not shutting down? Accept!
+			{/* Successfully selected, and still not shutting down? Accept! */
 				ssock = accept(msock, NULL, 0);
 			}
 			
 		} while ((msock > 0) && (ssock < 0)  && (time_to_die == 0));
 
 		if ((msock == -1)||(time_to_die))
-		{// ok, we're going down.
+		{/* ok, we're going down. */
 			int shutdown = 0;
 
 			/* the first to come here will have to do the cleanup.
@@ -991,18 +1045,17 @@ void worker_entry(void)
 			}
 			end_critical_section(S_SHUTDOWN);
 			if (shutdown == 1)
-			{// we're the one to cleanup the mess.
+			{/* we're the one to cleanup the mess. */
 				lprintf(2, "I'm master shutdown: tagging sessions to be killed.\n");
 				shutdown_sessions();
 				lprintf(2, "master shutdown: waiting for others\n");
-				sleeeeeeeeeep(1); // wait so some others might finish...
+				sleeeeeeeeeep(1); /* wait so some others might finish... */
 				lprintf(2, "master shutdown: cleaning up sessions\n");
 				do_housekeeping();
 				lprintf(2, "master shutdown: cleaning up libical\n");
-				free_zone_directory ();
-				icaltimezone_release_zone_tab ();
-				icalmemory_free_ring ();
-				ShutDownLibCitadel ();
+
+				ShutDownWebcit();
+
 				lprintf(2, "master shutdown exiting!.\n");				
 				exit(0);
 			}
@@ -1014,7 +1067,7 @@ void worker_entry(void)
 			if (ssock > 0) close (ssock);
 			lprintf(2, "inbetween.");
 			pthread_exit(NULL);
-		} else { // Got it? do some real work!
+		} else { /* Got it? do some real work! */
 			/* Set the SO_REUSEADDR socket option */
 			i = 1;
 			setsockopt(ssock, SOL_SOCKET, SO_REUSEADDR,
@@ -1033,7 +1086,7 @@ void worker_entry(void)
 			if (fail_this_transaction == 0) {
 
 				/* Perform an HTTP transaction... */
-				context_loop(ssock);
+				context_loop(&ssock);
 
 				/* Shut down SSL/TLS if required... */
 #ifdef HAVE_OPENSSL
@@ -1043,7 +1096,8 @@ void worker_entry(void)
 #endif
 
 				/* ...and close the socket. */
-				lingering_close(ssock);
+				if (ssock > 0)
+					lingering_close(ssock);
 			}
 
 		}
@@ -1055,11 +1109,12 @@ void worker_entry(void)
 }
 
 /*
- * \brief logprintf. log messages 
+ * print log messages 
  * logs to stderr if loglevel is lower than the verbosity set at startup
- * \param loglevel level of the message
- * \param format the printf like format string
- * \param ... the strings to put into format
+ *
+ * loglevel	level of the message
+ * format	the printf like format string
+ * ...		the strings to put into format
  */
 int lprintf(int loglevel, const char *format, ...)
 {
@@ -1076,7 +1131,7 @@ int lprintf(int loglevel, const char *format, ...)
 
 
 /*
- * \brief print the actual stack frame.
+ * print the actual stack frame.
  */
 void wc_backtrace(void)
 {
@@ -1098,4 +1153,3 @@ void wc_backtrace(void)
 #endif
 }
 
-/*@}*/
